/*
 * commands.h
 *
 *  Created on: 26.04.2013
 *      Author: AndreR
 */

#ifndef __COMMANDS_H__
#define __COMMANDS_H__

#include "stm32f10x.h"

#define PACKED __attribute__ ((packed))

#define PACKET_HEADER_SIZE 4
typedef union PACKED _PacketHeader
{
	struct PACKED
	{
		uint8_t cmd;
		uint8_t section;
		uint16_t len;   // data length only
	};

	uint8_t _data[PACKET_HEADER_SIZE];
} PacketHeader;

// SECTIONS
#define SECTION_CTRL 7
#define SECTION_BASE_STATION 8

// BASE_STATION
#define CMD_BASE_STATION_ACOMMAND 0	// this command is varying length, thus no LEN define
#define CMD_BASE_STATION_PING 1 // varying length too

#define CMD_BASE_STATION_AUTH 2

#define CMD_BASE_STATION_STATS 3

#define CMD_BASE_STATION_CONFIG 4

#define CMD_BASE_STATION_VISION_CONFIG 5

// CONTROL
#define CMD_CTRL_VISION_POS 0
#define LEN_CTRL_VISION_POS 6

typedef struct PACKED _BaseStationACommand
{
	uint8_t id;
	uint8_t* pCommand;
} BaseStationACommand;

typedef struct PACKED _BaseStationPing
{
	uint32_t id;
	uint8_t* pData;
} BaseStationPing;

typedef struct PACKED _BaseStationAuth	// 4
{
	uint32_t key;	// 0x42424242
} BaseStationAuth;

typedef struct PACKED _BaseStationStats // 60
{
	struct PACKED
	{
		uint32_t txBytesSent;
		uint32_t txBytesLost;
		uint32_t rxBytesRecv;
		uint32_t rxBytesLost;

		uint32_t txPacketsSent;
		uint32_t txPacketsLost;
		uint32_t rxPacketsRecv;
		uint32_t rxPacketsLost;

		uint32_t retransmissions;
	} wifi;

	struct PACKED
	{
		uint32_t txBytesSent;
		uint32_t rxBytesRecv;
		uint32_t rxBytesLost;

		uint32_t txFramesSent;
		uint32_t rxFramesRecv;
		uint32_t rxFramesLost;
	} eth;
} BaseStationStats;

// TODO: where to save base station config?
typedef struct PACKED _BaseStationConfig	// 7
{
	uint8_t channel;		// frequency
	uint8_t speed;
	uint8_t maxRt;
	uint16_t ackDelayUs;
	uint16_t maxVisionRate;
} BaseStationConfig;

typedef struct PACKED _BaseStationVisionConfig // 4
{
	uint8_t invertPosition;
	uint8_t blueTeam;
	uint16_t visionRate;
} BaseStationVisionConfig;

typedef struct PACKED _CtrlVisionPos	// 6
{
	int16_t pos[3];	// [mm], [mrad]
} CtrlVisionPos;

#endif /* COMMANDS_H_ */
