/*
 * config.c
 *
 *  Created on: 07.03.2013
 *      Author: AndreR
 */

#include "config.h"
#include "util/console.h"
#include "spi3.h"

#include <string.h>

#define EEPROM_MAGIC_BYTE 0x5D

Config config;

void ConfigInit()
{
	int16_t result = EEPROMInit(&config.eeprom, &spi3, GPIOA, GPIO_Pin_4);
	if(result)
		ConsolePrint("EEPROM init failed: 0x%0hX\r\n", result);

	result = EEPROMRead(&config.eeprom, 0, config._data, sizeof(config._data));
	if(result)
		ConsolePrint("EEPROM read failed: 0x%0hX\r\n", result);

	if(config.values.magic != EEPROM_MAGIC_BYTE)
	{
		// eeprom is not initialized
		ConsolePrint("EEPROM empty, programming defaults\r\n");

		ConfigReset();

		result = ConfigSave();
		if(result)
			ConsolePrint("Saving defaults failed: 0x%0hX\r\n", result);
	}
}

int16_t ConfigSave()
{
	int16_t result = EEPROMWrite(&config.eeprom, 0, config._data, sizeof(config._data));
	if(result)
		return result;

	return 0;
}

void ConfigReset()
{
	memset(config._data, 0, sizeof(config._data));
	config.values.magic = EEPROM_MAGIC_BYTE;

	config.values.ip= IPv4AddressSet(172, 31, 10, 210);
	config.values.channel = 100;
	config.values.speed = NRF24_SPEED_2M;
	config.values.maxRt = 3;
	config.values.ackDelayUs = 500;
	config.values.port = 10200;
	config.values.maxVisionRate = 30;
	config.values.maxNoLinkTries = 200;
}
