/*
 * core.c
 *
 *  Created on: 12.07.2010
 *      Author: rYan
 */

/*  .word  NMI_Handler
  .word  HardFault_Handler
  .word  MemManage_Handler
  .word  BusFault_Handler
  .word  UsageFault_Handler
  .word  0
  .word  0
  .word  0
  .word  0
  .word  SVC_Handler
  .word  DebugMon_Handler
*/

#include "stm32f10x.h"
#include "core_cm3.h"

void NMI_Handler()
{
//	asm volatile("MRS R4,MSP");			// read MSP (Main Stack Pointer)
	asm volatile("MRS R5,PSP");			// Read PSP	(Process Stack Pointer)
	asm volatile("LDR R0,[R5,#0]");
	asm volatile("LDR R1,[R5,#4]");
	asm volatile("LDR R2,[R5,#8]");
	asm volatile("LDR R3,[R5,#12]");
	asm volatile("LDR R12,[R5,#16]");	// R12
	asm volatile("LDR R11,[R5,#24]");	// R11 = PC
	asm volatile("LDR R10,[R5,#20]");	// R10 = LR

	SCB->AIRCR = 0x05FA0000 | (SCB->AIRCR & 0x700) | 0x4;	// System Reset, Keep priority grouping

	while(1) { asm volatile("nop"); }
}

void HardFault_Handler()
{
	asm volatile("MRS R0,PSP");			// Read PSP
	asm volatile("LDR R1,[R0,#24]");	// Read Saved PC from Stack

	while(1) { asm volatile("nop"); }
}

void MemManage_Handler()
{
	while(1) { asm volatile("nop"); }
}

void BusFault_Handler_C(uint32_t* args)
{
	uint32_t faultAddress = SCB->BFAR;

	uint32_t BFARVALID = SCB->CFSR & (1 << 15);
	uint32_t STKERR = SCB->CFSR & (1 << 12);
	uint32_t UNSTKERR = SCB->CFSR & (1 << 11);
	uint32_t IMPRECISERR = SCB->CFSR & (1 << 10);	// indicates invalid write operation
	uint32_t PRECISEERR = SCB->CFSR & (1 << 9);
	uint32_t IBUSERR = SCB->CFSR & (1 << 8);

	uint32_t r0 = args[0];
	uint32_t r1 = args[1];
	uint32_t r2 = args[2];
	uint32_t r3 = args[3];

	uint32_t r12 = args[4];
	uint32_t lr = args[5];
	uint32_t pc = args[6];
	uint32_t psr = args[7];

	while(1) { asm volatile("nop"); }
}

void UsageFault_Handler()
{
	uint32_t faultAddress = SCB->BFAR;

	uint32_t BFARVALID = SCB->CFSR & (1 << 15);
	uint32_t STKERR = SCB->CFSR & (1 << 12);
	uint32_t UNSTKERR = SCB->CFSR & (1 << 11);
	uint32_t IMORECISERR = SCB->CFSR & (1 << 10);
	uint32_t PRECISEERR = SCB->CFSR & (1 << 9);
	uint32_t IBUSERR = SCB->CFSR & (1 << 8);

//	asm volatile("MRS R4,MSP");			// read MSP (Main Stack Pointer)
	asm volatile("MRS R5,PSP");			// Read PSP	(Process Stack Pointer)
	asm volatile("LDR R0,[R5,#0]");
	asm volatile("LDR R1,[R5,#4]");
	asm volatile("LDR R2,[R5,#8]");
	asm volatile("LDR R3,[R5,#12]");
	asm volatile("LDR R12,[R5,#16]");	// R12
	asm volatile("LDR R11,[R5,#24]");	// R11 = PC
	asm volatile("LDR R10,[R5,#20]");	// R10 = LR

	while(1) { asm volatile("nop"); }
}

void DebugMon_Handler()
{
	while(1) { asm volatile("nop"); }
}
