/*
 * init.c
 *
 *  Created on: 17.05.2011
 *      Author: AndreR
 */

#include "init.h"

#include "stm32f10x_iwdg.h"

#include "FreeRTOS.h"
#include "task.h"

#include "cli.h"
#include "spi3.h"
#include "wifi.h"
#include "util/console.h"
#include "hal/usart1.h"
#include "hal/rtos_hf_timer.h"
#include "network.h"
#include "config.h"
#include "vision.h"

void InitTask(void* params)
{
	params = params;

	// Configure and start Independent watchdog
	IWDG_WriteAccessCmd(IWDG_WriteAccess_Enable);
	IWDG_SetPrescaler(IWDG_Prescaler_8);
	IWDG_SetReload(2500);	// approx. 500ms
	IWDG_WriteAccessCmd(IWDG_WriteAccess_Disable);

	// Initialize shared resources
	// --- HAL ---
	RTOSHFTimerInit();
	USART1Init();

	// --- GATEKEEPERS ---
	SPI3Init();

	// --- High-Level Systems ---
	CLIInit();

	// -- CLI
	ConsolePrint("\f\e[0;36mInitializing...\e[0m\r\n");

	uint32_t uid1 = *((uint32_t*)(0x1FFFF7E8 + 0));	// CPU ID register (96 bit)
	uint32_t uid2 = *((uint32_t*)(0x1FFFF7E8 + 4));
	uint32_t uid3 = *((uint32_t*)(0x1FFFF7E8 + 8));

	ConsolePrint("CPU ID: 0x%X 0x%X 0x%X\r\n", uid1, uid2, uid3);

	xTaskCreate(CLITask, (const signed char* const)"CLI", 512, NULL, 1, NULL);
	ConsolePrint("CLI: \e[0;32mOK\e[0m\r\n");

	// -- SPI3
	xTaskCreate(SPI3Task, (const signed char* const)"SPI", 256, NULL, 3, NULL);

	ConsolePrint("SPI 3: \e[0;32mOK\e[0m\r\n");

	ConfigInit();
	NetworkInit();
	WifiInit();
	VisionInit();

	// -- Ethernet
	ConsolePrint("Starting network\r\n");
	xTaskCreate(NetworkTask, (const signed char* const)"Net", 1536, NULL, 1, NULL);
	ConsolePrint("Done\r\n");


	// Create Tasks, MAX PRIORITY: 4
	xTaskCreate(WifiTask, (const signed char* const)"Wifi", 1024, NULL, 2, NULL);

	vTaskDelay(1000/portTICK_RATE_MS);

	ConsolePrint("%d Tasks started\r\n", uxTaskGetNumberOfTasks());
	ConsolePrint("HEAP usage: %d/%d Byte\r\n", configTOTAL_HEAP_SIZE-xPortGetFreeHeapSize(), configTOTAL_HEAP_SIZE);

//	Enable independent watchdog
//	IWDG_ReloadCounter();
//	IWDG_Enable();
//	ConsolePrint("Watchdog online\r\n");

	vTaskSuspend(NULL);	// Disable this task
}
