#include "stm32f10x.h"
#include "misc.h"
#include "stm32f10x_rcc.h"
#include "stm32f10x_exti.h"
#include "stm32f10x_gpio.h"
#include "stm32f10x_flash.h"
#include "stm32f10x_usart.h"
#include "stm32f10x_iwdg.h"

#include "core_cm3.h"

#include "util/bit_banding.h"

#include "FreeRTOS.h"
#include "task.h"

#include "cli.h"
#include "spi3.h"
#include "hal/led.h"
#include "util/console.h"

#include "hal/usart1.h"

#include "init.h"

void SystemInit()
{
	RCC_DeInit();

	RCC_HSEConfig(RCC_HSE_ON);
	while(RCC_GetFlagStatus(RCC_FLAG_HSERDY) == RESET);

	FLASH_PrefetchBufferCmd(FLASH_PrefetchBuffer_Enable);
	FLASH_SetLatency(FLASH_Latency_2);

	RCC_PREDIV1Config(RCC_PREDIV1_Source_HSE, RCC_PREDIV1_Div1);

	RCC_PLLConfig(RCC_PLLSource_PREDIV1, RCC_PLLMul_6);
	RCC_PLLCmd(ENABLE);

	while(RCC_GetFlagStatus(RCC_FLAG_PLLRDY) == RESET);

	RCC_HCLKConfig(RCC_SYSCLK_Div1);	// set AHB speed
	// setting APB2 to 72Mhz leads to some SPI problems (missing last bit)
	RCC_PCLK2Config(RCC_HCLK_Div2);		// set high speed APB2 (36Mhz)
	RCC_PCLK1Config(RCC_HCLK_Div2);		// set low speed APB1 (36Mhz)

	RCC_SYSCLKConfig(RCC_SYSCLKSource_PLLCLK);
	while(RCC_GetSYSCLKSource() != 0x08);

	SysTick_CLKSourceConfig(SysTick_CLKSource_HCLK);
}

int main()
{
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1 | RCC_AHBPeriph_DMA2, ENABLE);
	RCC_APB2PeriphClockCmd(	RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB | RCC_APB2Periph_GPIOC | RCC_APB2Periph_GPIOD |
							RCC_APB2Periph_AFIO, ENABLE);

	SCB->SHCSR |= SCB_SHCSR_USGFAULTENA | SCB_SHCSR_BUSFAULTENA | SCB_SHCSR_MEMFAULTENA;

	NVIC_SetVectorTable(NVIC_VectTab_FLASH, 0x00);
	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_4);

	LEDInit();

	LEDOn();

	xTaskCreate(InitTask, (const signed char* const)"Init", 512, NULL, 4, NULL);

	vTaskStartScheduler();

	return 0;
}

