/*
 * fifo_packet.c
 *
 *  Created on: 13.02.2011
 *      Author: AndreR
 */

#include "fifo_packet.h"

void FifoPacketInit(FifoPacket* pFifo, uint8_t* pBuf, uint16_t size)
{
	pFifo->numPackets = 0;
	FifoInit(&pFifo->fifo, pBuf, size);
}

void FifoPacketClear(FifoPacket* pFifo)
{
	FifoClear(&pFifo->fifo);
	pFifo->numPackets = 0;
}

int16_t FifoPacketPush(FifoPacket* pFifo, uint16_t packetLength, uint8_t* pData)
{
	if(packetLength + FIFO_PACKET_HEADER_SIZE > pFifo->fifo.EmptyBytes)
		return ERROR_FIFO_PACKET_NOT_ENOUGH_MEMORY;

	FifoPush(&pFifo->fifo, (uint8_t)packetLength);
	FifoPush(&pFifo->fifo, (uint8_t)(packetLength >> 8));

	for(uint16_t i = 0; i < packetLength; i++)
		FifoPush(&pFifo->fifo, pData[i]);

	pFifo->numPackets++;

	return 0;
}

int16_t FifoPacketPop(FifoPacket* pFifo, uint16_t* pPacketLength, uint8_t* pData, uint16_t suppliedLength)
{
	if(pFifo->numPackets == 0)
		return ERROR_FIFO_PACKET_EMPTY;

	FifoPacketGetNextPacketDataLength(pFifo, pPacketLength);

	if(suppliedLength < *pPacketLength)
		return ERROR_FIFO_PACKET_NOT_ENOUGH_MEMORY;

	FifoPop(&pFifo->fifo, ((uint8_t*)pPacketLength)+0);
	FifoPop(&pFifo->fifo, ((uint8_t*)pPacketLength)+1);

	for(uint16_t i = 0; i < *pPacketLength; i++)
		FifoPop(&pFifo->fifo, pData+i);

	pFifo->numPackets--;

	return 0;
}

int16_t FifoPacketDrop(FifoPacket* pFifo)
{
	if(pFifo->numPackets == 0)
		return ERROR_FIFO_PACKET_EMPTY;

	uint16_t len;

	FifoPop(&pFifo->fifo, ((uint8_t*)&len)+0);
	FifoPop(&pFifo->fifo, ((uint8_t*)&len)+1);

	uint8_t lost;

	for(uint16_t i = 0; i < len; i++)
		FifoPop(&pFifo->fifo, &lost);

	pFifo->numPackets--;

	return 0;
}

int16_t FifoPacketGetNextPacketDataLength(FifoPacket* pFifo, uint16_t* pLength)
{
	if(pFifo->numPackets == 0)
		return ERROR_FIFO_PACKET_EMPTY;

	//save read state
	volatile uint8_t* pRead = pFifo->fifo.pRead;

	FifoPop(&pFifo->fifo, ((uint8_t*)pLength)+0);
	FifoPop(&pFifo->fifo, ((uint8_t*)pLength)+1);

	pFifo->fifo.pRead = pRead;
	pFifo->fifo.EmptyBytes -= FIFO_PACKET_HEADER_SIZE;

	return 0;
}

uint16_t FifoPacketNumPackets(FifoPacket* pFifo)
{
	return pFifo->numPackets;
}
