/*
 * linked_list.c
 *
 *  Created on: 27.04.2013
 *      Author: AndreR
 */

#include "linked_list.h"

void LinkedListInit(LinkedList* pList)
{
	pList->pHead = 0;
	pList->pTail = 0;
}

void LinkedListPushBack(LinkedList* pList, void* pData)
{
	LinkedListEntry* pEntry = (LinkedListEntry*)pData;
	pEntry->pNext = 0;
	pEntry->pPrevious = 0;

	if(pList->pHead == 0) // means that pList->pTail is also 0
	{
		pList->pHead = pEntry;
		pList->pTail = pEntry;
		return;
	}

	pEntry->pPrevious = pList->pTail;
	pList->pTail->pNext = pEntry;
	pList->pTail = pEntry;
}

void* LinkedListPopFront(LinkedList* pList)
{
	if(pList->pHead == 0)
		return 0;

	LinkedListEntry* pFront = pList->pHead;

	if(pList->pHead == pList->pTail)
		pList->pTail = pList->pTail->pNext;

	pList->pHead = pList->pHead->pNext;

	pFront->pPrevious = 0;
	pFront->pNext = 0;

	return pFront;
}

void* LinkedListFind(LinkedList* pList, LinkedListCompareFunc pCompare, void* pUserData)
{
	for(LinkedListEntry* pTmp = pList->pHead; pTmp->pNext != 0; pTmp = pTmp->pNext)
	{
		if((*pCompare)(pTmp, pUserData))
			return pTmp;
	}

	return 0;
}

void LinkedListRemove(LinkedList* pList, void* pData)
{
	LinkedListEntry* pEntry = (LinkedListEntry*)pData;

	if(pEntry->pPrevious)
		pEntry->pPrevious->pNext = pEntry->pNext;

	if(pEntry->pNext)
		pEntry->pNext->pPrevious = pEntry->pPrevious;

	if(pList->pHead == pEntry)
		pList->pHead = pEntry->pNext;

	if(pList->pTail == pEntry)
		pList->pTail = pEntry->pPrevious;

	pEntry->pPrevious = 0;
	pEntry->pNext = 0;
}
