/*
 * vision.c
 *
 *  Created on: 22.05.2013
 *      Author: AndreR
 */

#include "vision.h"
#include "util/console.h"
#include "network.h"
#include "wifi.h"
#include <string.h>

Vision vision;

static void* allocMem(void *allocator_data, size_t size)
{
	(void)allocator_data;

	if(vision.allocUsed + size > VISION_ALLOC_BUF_SIZE)
	{
		ConsolePrint("WARN: vision out of memory\r\n");
		return 0;
	}

	void* pMem = vision.pAllocBuf + vision.allocUsed;

	vision.allocUsed += size;

	return pMem;
}

static void freeMem(void *allocator_data, void *pointer)
{
	(void)allocator_data;
	(void)pointer;
}

#define PI 3.14159265359f

static int16_t visionCallback(uint8_t* pData, uint16_t length)
{
//	ConsolePrint("vision len: %u\r\n", length);

	// TODO: testing, no vision input
//	return 0;

	VisionFree();
	SSLWrapperPacket* pPacket = 0;
	pPacket = VisionUnpack(pData, length);
	if(pPacket == 0)
		return 0;

	if(pPacket->detection == 0)
		return 0;

	uint32_t numRobots = pPacket->detection->n_robots_yellow;
	SSLDetectionRobot** ppRobots = pPacket->detection->robots_yellow;
	if(vision.blueTeam)
	{
		numRobots = pPacket->detection->n_robots_blue;
		ppRobots = pPacket->detection->robots_blue;
	}

//	ConsolePrint("TIGERS numBots: %u\r\n", numRobots);

	for(uint32_t i = 0; i < numRobots; i++)
	{
		SSLDetectionRobot* pRobot = ppRobots[i];

		if(pRobot->has_robot_id == 0 || pRobot->has_orientation == 0)
			continue;

		if(vision.invertPos)
		{
			pRobot->x *= -1.0f;
			pRobot->y *= -1.0f;
			pRobot->orientation += PI;
			if(pRobot->orientation > PI)
				pRobot->orientation -= 2*PI;
			if(pRobot->orientation < -PI)
				pRobot->orientation += 2*PI;
		}

		WifiSendPose(pRobot->robot_id, pRobot->x, pRobot->y, pRobot->orientation);

//		ConsolePrint("ID: %u, Pos: (%d,%d)\r\n", pRobot->robot_id, (int32_t)pRobot->x, (int32_t)pRobot->y);
	}

	return 0;
}

void VisionInit()
{
	memset(&vision, 0, sizeof(Vision));

	protobuf_c_default_allocator.alloc = &allocMem;
	protobuf_c_default_allocator.free = &freeMem;
	protobuf_c_default_allocator.max_alloca = VISION_ALLOC_BUF_SIZE;
	protobuf_c_default_allocator.tmp_alloc = 0;
	protobuf_c_default_allocator.allocator_data = 0;

	NetworkSetVisionCallback(&visionCallback);
}

void VisionFree()
{
	vision.allocUsed = 0;
}

SSLWrapperPacket* VisionUnpack(void* pData, size_t len)
{
	return ssl__wrapper_packet__unpack(0, len, pData);
}
