ifndef TARGET
$(error No TARGET specified)
endif

include $(TARGET)

# STM variables
CC = $(TOOLCHAIN_PATH)/bin/arm-none-eabi-gcc $(TARGET_ARCH)
CFLAGS = -std=gnu99 -ffunction-sections -fdata-sections $(WARNINGS) $(TARGET_OPTS) $(MESSAGES) $(F_NO_STRICT_ALIASING) $(INCLUDE_DIRS)

CXX = $(TOOLCHAIN_PATH)/bin/arm-none-eabi-g++ $(TARGET_ARCH)
CXXFLAGS = -std=gnu++98 $(WARNINGS) $(TARGET_OPTS) $(MESSAGES) $(INCLUDE_DIRS) $(WARNINGS_CXX) $(F_NO_EXCEPTIONS)

AS = $(CC) -x assembler-with-cpp -c $(TARGET_ARCH)
ASFLAGS = $(WARNINGS) $(TARGET_OPTS) $(MESSAGES) $(INCLUDE_DIRS)


LD = $(CC)	#C++ variant: LD = $(CXX)
LDFLAGS = -Wl,-Map=$(MAIN_MAP),--gc-sections -Tstm32.ld -nostartfiles -static $(LIBRARY_DIRS) $(LIBRARY_DIRS)

AR = $(TOOLCHAIN_PATH)/bin/arm-none-eabi-ar
ARFLAGS = cr

OBJCOPY = $(TOOLCHAIN_PATH)/bin/arm-none-eabi-objcopy
OBJCOPYFLAGS = -O binary

OBJDUMP = $(TOOLCHAIN_PATH)/bin/arm-none-eabi-objdump
OBJDUMPFLAGS = -h -t -S -C -D -l $(addprefix -I,$(SRC_DIRS)) -Ifwlib/src

SIZE = $(TOOLCHAIN_PATH)/bin/arm-none-eabi-size

MAIN_MAP = $(MAIN_OUT:%.elf=%.map)
MAIN_BIN = $(MAIN_OUT:%.elf=%.bin)
MAIN_LSS = $(MAIN_OUT:%.elf=%.lss)

SRC_DIRS = $(addprefix src/,$(SOURCE_DIRS))
STATIC_SRC_DIRS = $(addprefix src/,$(STATIC_SOURCE_DIRS))
ALL_SRC_DIRS = $(SRC_DIRS) $(STATIC_SRC_DIRS)

OBJS_C = $(addprefix $(BUILD_DIR)/,$(patsubst %.c,%.o,$(wildcard $(addsuffix /*.c,$(SRC_DIRS)))))
OBJS_S = $(addprefix $(BUILD_DIR)/,$(patsubst %.s,%.o,$(wildcard $(addsuffix /*.s,$(SRC_DIRS)))))
OBJS_CPP = $(addprefix $(BUILD_DIR)/,$(patsubst %.cpp,%.o,$(wildcard $(addsuffix /*.cpp,$(SRC_DIRS)))))
OBJS = $(OBJS_C) $(OBJS_CPP) $(OBJS_S)

STATIC_OBJS_C = $(addprefix $(BUILD_DIR)/,$(patsubst %.c,%.o,$(wildcard $(addsuffix /*.c,$(STATIC_SRC_DIRS)))))
STATIC_OBJS_S = $(addprefix $(BUILD_DIR)/,$(patsubst %.s,%.o,$(wildcard $(addsuffix /*.s,$(STATIC_SRC_DIRS)))))
STATIC_OBJS_CPP = $(addprefix $(BUILD_DIR)/,$(patsubst %.cpp,%.o,$(wildcard $(addsuffix /*.cpp,$(STATAIC_SRC_DIRS)))))
STATIC_OBJS = $(STATIC_OBJS_C) $(STATIC_OBJS_CPP) $(STATIC_OBJS_S)

ALL_OBJS = $(STATIC_OBJS) $(OBJS)

#############
### RULES ###
#############

# STM
build: $(MAIN_OUT)
	$(OBJCOPY) $(OBJCOPYFLAGS) $(MAIN_OUT) $(MAIN_BIN)
#	$(OBJDUMP) $(OBJDUMPFLAGS) $(MAIN_OUT) > $(MAIN_LSS)
	$(SIZE) $<

$(MAIN_OUT): folders $(ALL_OBJS)
	$(LD) $(LDFLAGS) $(TARGET_ARCH) $(AVR_OBJS) $(ALL_OBJS) $(LIBRARIES) -o $@
	
-include $(ALL_OBJS:.o=.d)
	
define MAKE_IMPLICIT_C
$(BUILD_DIR)/$(1)/%.o: $(1)/%.c
	$(CC) $(CFLAGS) -c $$< -o $$@
	@$(CC) -MM $(CFLAGS) $$< > $(BUILD_DIR)/$(1)/$$*.d.tmp
	@$(SED) -e "s|.*:|$$@:|" < $(BUILD_DIR)/$(1)/$$*.d.tmp > $(BUILD_DIR)/$(1)/$$*.d
	@$(SED) -e "s|.*:||" -e "s|\\\\||" < $(BUILD_DIR)/$(1)/$$*.d.tmp | $(FMT) -1 | \
	  $(SED) -e "s|^ *||" -e "s|$$$$|:|" >> $(BUILD_DIR)/$(1)/$$*.d
	@$(RM) -f $(BUILD_DIR)/$(1)/$$*.d.tmp
endef

define MAKE_IMPLICIT_S
$(BUILD_DIR)/$(1)/%.o: $(1)/%.s
	$(AS) $(ASFLAGS) -c $$< -o $$@
	@$(CC) -MM $(CFLAGS) $$< > $(BUILD_DIR)/$(1)/$$*.d.tmp
	@$(SED) -e "s|.*:|$$@:|" < $(BUILD_DIR)/$(1)/$$*.d.tmp > $(BUILD_DIR)/$(1)/$$*.d
	@$(SED) -e "s|.*:||" -e "s|\\\\||" < $(BUILD_DIR)/$(1)/$$*.d.tmp | $(FMT) -1 | \
	  $(SED) -e "s|^ *||" -e "s|$$$$|:|" >> $(BUILD_DIR)/$(1)/$$*.d
	@$(RM) -f $(BUILD_DIR)/$(1)/$$*.d.tmp
endef

define MAKE_IMPLICIT_CPP
$(BUILD_DIR)/$(1)/%.o: $(1)/%.cpp
	$(CXX) $(CXXFLAGS) -c $$< -o $$@
	@$(CC) -MM $(CFLAGS) $$^ > $(BUILD_DIR)/$(1)/$$*.d.tmp
	@$(SED) -e "s|.*:|$$@:|" < $(BUILD_DIR)/$(1)/$$*.d.tmp > $(BUILD_DIR)/$(1)/$$*.d
	@$(SED) -e "s|.*:||" -e "s|\\\\||" < $(BUILD_DIR)/$(1)/$$*.d.tmp | $(FMT) -1 | \
	  $(SED) -e "s|^ *||" -e "s|$$$$|:|" >> $(BUILD_DIR)/$(1)/$$*.d
	@$(RM) -f $(BUILD_DIR)/$(1)/$$*.d.tmp
endef

$(foreach dir, $(ALL_SRC_DIRS), $(eval $(call MAKE_IMPLICIT_C,$(dir))))
$(foreach dir, $(ALL_SRC_DIRS), $(eval $(call MAKE_IMPLICIT_S,$(dir))))
$(foreach dir, $(ALL_SRC_DIRS), $(eval $(call MAKE_IMPLICIT_CPP,$(dir))))

# Stuff
.PHONY: folders
folders:
	$(MKDIR) -p $(addprefix $(BUILD_DIR)/, $(ALL_SRC_DIRS))

.PHONY: dist-clean
dist-clean: clean
	-$(RM) -f -r $(MAIN_OUT) $(MAIN_MAP) $(MAIN_BIN) $(MAIN_LSS) $(BUILD_DIR)
	
.PHONY: clean
clean:
	-$(RM) -f $(OBJS) $(OBJS:.o=.d)
