/*
 * power.c
 *
 * Power control and overcurrent watchdog.
 *
 *  Created on: 25.10.2010
 *      Author: AndreR
 */

#include "power.h"

#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_iwdg.h"

#include "FreeRTOS.h"
#include "task.h"

#include "commands.h"
#include "network.h"
#include "hal/buzzer.h"
#include "hal/led.h"
#include "util/console.h"

#include "spi1_2_3.h"

#define POWER_5V_ENABLE (1 << 12)	// GPIOB

void PowerPreInit()
{
	GPIO_InitTypeDef gpioInit;

	gpioInit.GPIO_Mode = GPIO_Mode_OUT;
	gpioInit.GPIO_Pin = POWER_5V_ENABLE;
	gpioInit.GPIO_Speed = GPIO_Speed_2MHz;
	gpioInit.GPIO_OType = GPIO_OType_PP;
	GPIO_Init(GPIOB, &gpioInit);
}

void PowerInit()
{
}

void PowerEnable5VSupply(uint8_t enable)
{
	if(enable)
		GPIOB->BSRRH = POWER_5V_ENABLE;
	else
		GPIOB->BSRRL = POWER_5V_ENABLE;
}

uint8_t PowerIs5VSupplyEnabled()
{
	if(GPIOB->ODR & POWER_5V_ENABLE)
		return 0;
	else
		return 1;
}

void PowerDummyTask(void* params)
{
	(void)params;

	while(1)
	{
		SystemPowerLog log;
		log.iVCC = 0;
		log.i5V = 0;
		log.i3V3 = 0;
		log.u1 = 3700;
		log.u2 = 3700;
		log.u3 = 3700;
		log.u4 = 3700;

		PacketHeader header;
		header.section = SECTION_SYSTEM;
		header.cmd = CMD_SYSTEM_POWER_LOG;
		header.len = LEN_SYSTEM_POWER_LOG;

		NetworkSendPacket(&header, (uint8_t*)&log);

		vTaskDelay(500/portTICK_RATE_MS);
	}
}
