/*
 * spi1_2_3.c
 *
 *  Created on: 06.03.2011
 *      Author: AndreR
 */

#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "misc.h"

#include "spi1_2_3.h"
#include "constants.h"

#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

SPI spi1;
SPI spi2;
SPI spi3;

xSemaphoreHandle hNewPacket;

void DMA2_Stream0_IRQHandler()
{
	SPIDmaRxInterrupt(&spi1);
}

void DMA1_Stream3_IRQHandler()
{
	SPIDmaRxInterrupt(&spi2);
}

void DMA1_Stream0_IRQHandler()
{
	SPIDmaRxInterrupt(&spi3);
}

void SPI1Callback()
{
	signed portBASE_TYPE xHigherPriorityTaskWoken = pdFALSE;

	xSemaphoreGiveFromISR(hNewPacket, &xHigherPriorityTaskWoken);

	if(xHigherPriorityTaskWoken == pdTRUE)
		taskYIELD();
}

void SPI2Callback()
{
	signed portBASE_TYPE xHigherPriorityTaskWoken = pdFALSE;

	xSemaphoreGiveFromISR(hNewPacket, &xHigherPriorityTaskWoken);

	if(xHigherPriorityTaskWoken == pdTRUE)
		taskYIELD();
}

void SPI3Callback()
{
	signed portBASE_TYPE xHigherPriorityTaskWoken = pdFALSE;

	xSemaphoreGiveFromISR(hNewPacket, &xHigherPriorityTaskWoken);

	if(xHigherPriorityTaskWoken == pdTRUE)
		taskYIELD();
}

int16_t SPI1_2_3Init()
{
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI2, ENABLE);
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI3, ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_SPI1, ENABLE);

	static uint8_t spi1TxData[SPI1_FIFO_SIZE];
	static uint8_t spi1RxData[SPI1_FIFO_SIZE];
	static uint8_t spi1DmaTxData[SPI1_DMA_SIZE];
	static uint8_t spi1DmaRxData[SPI1_DMA_SIZE];
	static uint8_t spi1CbData[SPI1_CB_SIZE];

	static uint8_t spi2TxData[SPI2_FIFO_SIZE];
	static uint8_t spi2RxData[SPI2_FIFO_SIZE];
	static uint8_t spi2DmaTxData[SPI2_DMA_SIZE];
	static uint8_t spi2DmaRxData[SPI2_DMA_SIZE];
	static uint8_t spi2CbData[SPI2_CB_SIZE];

	static uint8_t spi3TxData[SPI3_FIFO_SIZE];
	static uint8_t spi3RxData[SPI3_FIFO_SIZE];
	static uint8_t spi3DmaTxData[SPI3_DMA_SIZE];
	static uint8_t spi3DmaRxData[SPI3_DMA_SIZE];
	static uint8_t spi3CbData[SPI3_CB_SIZE];

	SPIData data1;
	data1.pDataTx = spi1TxData;
	data1.pDataRx = spi1RxData;
	data1.size = SPI1_FIFO_SIZE;
	data1.pDmaDataTx = spi1DmaTxData;
	data1.pDmaDataRx = spi1DmaRxData;
	data1.dmaSize = SPI1_DMA_SIZE;
	data1.pCbData = spi1CbData;
	data1.cbSize = SPI1_CB_SIZE;
	data1.dmaChRx = 0;
	data1.dmaChTx = 3;
	data1.pDma = DMA2;

	SPIData data2;
	data2.pDataTx = spi2TxData;
	data2.pDataRx = spi2RxData;
	data2.size = SPI2_FIFO_SIZE;
	data2.pDmaDataTx = spi2DmaTxData;
	data2.pDmaDataRx = spi2DmaRxData;
	data2.dmaSize = SPI2_DMA_SIZE;
	data2.pCbData = spi2CbData;
	data2.cbSize = SPI2_CB_SIZE;
	data2.dmaChRx = 3;
	data2.dmaChTx = 4;
	data2.pDma = DMA1;

	SPIData data3;
	data3.pDataTx = spi3TxData;
	data3.pDataRx = spi3RxData;
	data3.size = SPI3_FIFO_SIZE;
	data3.pDmaDataTx = spi3DmaTxData;
	data3.pDmaDataRx = spi3DmaRxData;
	data3.dmaSize = SPI3_DMA_SIZE;
	data3.pCbData = spi3CbData;
	data3.cbSize = SPI3_CB_SIZE;
	data3.dmaChRx = 0;
	data3.dmaChTx = 7;
	data3.pDma = DMA1;

	SPIInit(&spi1, SPI1, data1, &SPI1Callback);
	SPIInit(&spi2, SPI2, data2, &SPI2Callback);
	SPIInit(&spi3, SPI3, data3, &SPI3Callback);

	NVIC_InitTypeDef NVICInit;
	NVICInit.NVIC_IRQChannel = DMA2_Stream0_IRQn;
	NVICInit.NVIC_IRQChannelCmd = ENABLE;
	NVICInit.NVIC_IRQChannelPreemptionPriority = IRQL_SPI1;
	NVICInit.NVIC_IRQChannelSubPriority = 0x00;
	NVIC_Init(&NVICInit);

	NVICInit.NVIC_IRQChannel = DMA1_Stream3_IRQn;
	NVICInit.NVIC_IRQChannelPreemptionPriority = IRQL_SPI2;
	NVIC_Init(&NVICInit);

	NVICInit.NVIC_IRQChannel = DMA1_Stream0_IRQn;
	NVICInit.NVIC_IRQChannelPreemptionPriority = IRQL_SPI3;
	NVIC_Init(&NVICInit);

	GPIO_InitTypeDef gpioInit;
	gpioInit.GPIO_Mode = GPIO_Mode_AF;
	gpioInit.GPIO_OType = GPIO_OType_PP;
	gpioInit.GPIO_PuPd = GPIO_PuPd_UP;
	gpioInit.GPIO_Pin = GPIO_Pin_5 | GPIO_Pin_6 | GPIO_Pin_7;	// SCK, MISO, MOSI
	gpioInit.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOA, &gpioInit);

	gpioInit.GPIO_Pin = GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
	GPIO_Init(GPIOB, &gpioInit);

	gpioInit.GPIO_Pin = GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12;
	GPIO_Init(GPIOC, &gpioInit);

	GPIO_PinAFConfig(GPIOA, GPIO_PinSource5, GPIO_AF_SPI1);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource6, GPIO_AF_SPI1);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource7, GPIO_AF_SPI1);

	GPIO_PinAFConfig(GPIOB, GPIO_PinSource13, GPIO_AF_SPI2);
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource14, GPIO_AF_SPI2);
	GPIO_PinAFConfig(GPIOB, GPIO_PinSource15, GPIO_AF_SPI2);

	GPIO_PinAFConfig(GPIOC, GPIO_PinSource10, GPIO_AF_SPI3);
	GPIO_PinAFConfig(GPIOC, GPIO_PinSource11, GPIO_AF_SPI3);
	GPIO_PinAFConfig(GPIOC, GPIO_PinSource12, GPIO_AF_SPI3);

	vSemaphoreCreateBinary(hNewPacket);

	return 0;
}

void SPI1_2_3Task(void* params)
{
	params = params;

	while(1)
	{
		xSemaphoreTake(hNewPacket, 10/portTICK_RATE_MS);

		SPINotifySlaveCallbacks(&spi1);
		SPINotifySlaveCallbacks(&spi2);
		SPINotifySlaveCallbacks(&spi3);
	}
}
