/*
 * bootloader.c
 *
 *  Created on: 03.06.2013
 *      Author: AndreR
 */

#include "bootloader.h"

#include "intercom_constants.h"
#include "intercom1.h"
#include "commands.h"
#include "util/console.h"
#include "util/boot.h"

static void sendACK(uint32_t offset)
{
	BootloaderResponse response;
	response.type = BOOTLOADER_RESPONSE_ACK;
	response.offset = offset;

	IntercomHeader header;
	header.address = INTERCOM_ADDRESS_MAIN;
	header.section = INTERCOM_SECTION_BOOTLOADER;
	header.command = BOOTLOADER_RESPONSE;

	IntercomSend(&intercom1, &header, sizeof(BootloaderResponse), (uint8_t*)&response);
}

static void bootloaderCb(IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData, void* pUserData)
{
	(void)pUserData;
	(void)dataLength;

	switch(pHeader->command)
	{
		case BOOTLOADER_ERASE:
		{
			ConsolePrint("Erasing media\r\n");

			if(BootEraseProgramFlash())
				ConsolePrint("Erase failed\r\n");

			sendACK(0);
		}
		break;
		case BOOTLOADER_DATA:
		{
			BootloaderData* pBootData = (BootloaderData*)pData;

			BootProgramFlash(pBootData->data, 64, pBootData->offset);

			sendACK(pBootData->offset);
		}
		break;
	}
}

void BootloaderInit()
{
	IntercomClient client;
	client.pUserData = 0;
	client.section = INTERCOM_SECTION_BOOTLOADER;
	client.rxCallback = &bootloaderCb;

	IntercomAddClient(&intercom1, client);
}
