/*
 * cli.c
 *
 *  Created on: 19.11.2012
 *      Author: AndreR
 */

#include "cli.h"

#include "util/console.h"
#include "intercom_constants.h"
#include "hal/led.h"
#include "hal/usart2.h"
#include "hal/rtos_hf_timer.h"
#include "intercom1.h"
#include "spi3.h"

#include "FreeRTOS.h"
#include "task.h"

CLI cli;

void processCommand();

void CLIInit()
{
	cli.hProcBufMutex = xSemaphoreCreateRecursiveMutex();

	static uint8_t rxData[CONSOLE_RX_SIZE];
	static uint8_t txData[CONSOLE_TX_SIZE];
	static uint8_t printData[CONSOLE_PRINT_SIZE];
	static uint8_t scanData[CONSOLE_SCAN_SIZE];

	ConsoleData conData;
	conData.pDataTx = txData;
	conData.txSize = CONSOLE_TX_SIZE;
	conData.pDataRx = rxData;
	conData.rxSize = CONSOLE_RX_SIZE;
	conData.pPrintBuf = printData;
	conData.printSize = CONSOLE_PRINT_SIZE;
	conData.pScanBuf = scanData;
	conData.scanSize = CONSOLE_SCAN_SIZE;
	conData.txCallback = 0;
	conData.pUserData = 0;

	ConsoleInit(conData);

	static uint8_t adapterBuf[CONSOLE_INTERCOM_ADAPTER_BUF_SIZE];

	ConsoleIntercomAdapterInit(&cli.adapter, &intercom1, adapterBuf,
			CONSOLE_INTERCOM_ADAPTER_BUF_SIZE, INTERCOM_ADDRESS_MEDIA, INTERCOM_ADDRESS_MAIN);
}

void CLITask(void* params)
{
	params = params;

	portTickType xLastWakeTime;
	portTickType xLastLEDTime;

	xLastWakeTime = xTaskGetTickCount();
	xLastLEDTime = xLastWakeTime;

	LEDMustOff(LED_MAIN_GREEN);

	while(1)
	{
		if(ConsoleIsCmdReady())
		{
			processCommand();

			ConsoleCmdProcessed();
		}

		// Heartbeat LED
		if(xLastWakeTime - xLastLEDTime > 500/portTICK_RATE_MS)
		{
			xLastLEDTime = xLastWakeTime;

			LEDToggle(LED_MAIN_RED);
		}

		vTaskDelayUntil(&xLastWakeTime, 50/portTICK_RATE_MS);
	}
}

void processCommand()
{
	if(ConsoleCmpCmd("test"))
	{
		ConsolePrint("Number: %d\r\n", 43);
	}

	if(ConsoleCmpCmd("test2"))
	{
		ConsolePrint("0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789\r\n");
	}

	if(ConsoleCmpCmd("task list"))
	{
		char* printBuf = ConsoleGetPrintBuf();

		ConsolePrint("\r\nTask\t\tState\tPrio\tStack\tID");

		ConsoleLockPrintBuf();
		vTaskList((signed char*)printBuf);
		ConsoleSendPrintBuf();
		ConsoleUnlockPrintBuf();
	}

	if(ConsoleCmpCmd("task stats"))
	{
		char* printBuf = ConsoleGetPrintBuf();

		ConsolePrint("\r\nTask\t\tTicks\t\tRatio");

		ConsoleLockPrintBuf();
		vTaskGetRunTimeStats((signed char*)printBuf);
		ConsoleSendPrintBuf();
		ConsoleUnlockPrintBuf();
	}

	if(ConsoleCmpCmd("usart2 state"))
	{
		ConsolePrint("TX: %d\r\n", usart2.txState);
	}
}
