/*
 * usart2.c
 *
 *  Created on: 03.11.2012
 *      Author: AndreR
 */

#include "usart2.h"

#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_usart.h"
#include "stm32f4xx_dma.h"
#include "stm32f4xx_syscfg.h"
#include "stm32f4xx_exti.h"
#include "misc.h"
#include "hal/exti.h"

USART usart2;

static void ext1IRQ()
{
	USARTTxSyncFalling(&usart2);
}

void USART2_IRQHandler()
{
	USARTIRQ(&usart2);
}

void DMA1_Stream5_IRQHandler()
{
	USARTDMARx(&usart2);
}

// USART2 TX
void DMA1_Stream6_IRQHandler()
{
	USARTDMATx(&usart2);
}

void USART2Init()
{
	// enable peripheral clock
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART2, ENABLE);

	// provide data storage
	static uint8_t usart2TxData[USART2_FIFO_SIZE];
	static uint8_t usart2RxData[USART2_FIFO_SIZE];
	static uint8_t usart2DmaTxData[USART2_DMA_SIZE];
	static uint8_t usart2DmaRxData[USART2_DMA_SIZE];

	// map data and configure
	USARTData data2;
	data2.pDataRx = usart2RxData;
	data2.pDataTx = usart2TxData;
	data2.size = USART2_FIFO_SIZE;
	data2.pDmaDataRx = usart2DmaRxData;
	data2.pDmaDataTx = usart2DmaTxData;
	data2.dmaSize = USART2_DMA_SIZE;
	data2.pDMA = DMA1;
	data2.dmaChannel = DMA_Channel_4;
	data2.rxStream = 5;
	data2.txStream = 6;
	data2.rxIRQn = DMA1_Stream5_IRQn;
	data2.txIRQn = DMA1_Stream6_IRQn;
	data2.usartIRQn = USART2_IRQn;
	data2.IRQL = IRQL_USART2;
	data2.baudrate = 5250000;
	data2.pTxSyncPort = GPIOB;
	data2.txSyncPin = GPIO_Pin_14;	// EXT1
	data2.pRxSyncPort = GPIOB;
	data2.rxSyncPin = GPIO_Pin_0;	// EXT2

	// Init USART util layer
	USARTInit(&usart2, USART2, data2);

	// GPIO setup
	GPIO_InitTypeDef gpioInit;
	gpioInit.GPIO_Mode = GPIO_Mode_AF;
	gpioInit.GPIO_OType = GPIO_OType_PP;
	gpioInit.GPIO_Pin = GPIO_Pin_2 | GPIO_Pin_3;
	gpioInit.GPIO_Speed = GPIO_Speed_50MHz;
	gpioInit.GPIO_PuPd = GPIO_PuPd_UP;
	GPIO_Init(GPIOA, &gpioInit);

	// Map alternate function
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource2, GPIO_AF_USART2);
	GPIO_PinAFConfig(GPIOA, GPIO_PinSource3, GPIO_AF_USART2);

	SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOB, EXTI_PinSource14);

	EXTI_InitTypeDef extiInit;
	EXTI_StructInit(&extiInit);
	extiInit.EXTI_Line = EXTI_Line14;
	extiInit.EXTI_LineCmd = ENABLE;
	extiInit.EXTI_Mode = EXTI_Mode_Interrupt;
	extiInit.EXTI_Trigger = EXTI_Trigger_Falling;
	EXTI_Init(&extiInit);

	ExtiSetCallback(&ext1IRQ, 14);
}
