/*
 * console_intercom_adapter.c
 *
 *  Created on: 16.11.2012
 *      Author: AndreR
 */

#include "console_intercom_adapter.h"

#include "intercom_constants.h"

static void intercomCallback(IntercomHeader* pHeader, uint16_t dataLength, uint8_t* pData, void* pUserData)
{
	ConsoleIntercomAdapter* pAdapter = (ConsoleIntercomAdapter*)pUserData;

	int16_t result = FifoPacketSyncedBegin(&console.rxFifo);
	if(result)
	{
		pAdapter->droppedPackets++;
		return;
	}

	result = FifoPacketSyncedPush(&console.rxFifo, dataLength, pData);
	if(result)
	{
		FifoPacketSyncedCancel(&console.rxFifo);
		pAdapter->droppedPackets++;
		return;
	}

	FifoPacketSyncedComplete(&console.rxFifo);
}

static void consoleTxCallback(void* pUserData)
{
	ConsoleIntercomAdapter* pAdapter = (ConsoleIntercomAdapter*)pUserData;

	IntercomHeader header;
	header.address = pAdapter->intercomDestination;
	header.command = pAdapter->intercomSource;
	header.section = INTERCOM_SECTION_CONSOLE;

	while(console.txFifo.numPackets > 0)
	{
		uint16_t length;
		int16_t result = FifoPacketSyncedPop(&console.txFifo, &length, pAdapter->pTransferBuf, pAdapter->transferBufSize);
		if(result == ERROR_FIFO_PACKET_SYNCED_NOT_ENOUGH_MEMORY)
		{
			FifoPacketSyncedDrop(&console.txFifo);
			pAdapter->droppedPackets++;
			continue;
		}

		if(result)
			return;

		result = IntercomSend(pAdapter->pIntercom, &header, length, pAdapter->pTransferBuf);
		if(result)
			pAdapter->droppedPackets++;
	}
}

void ConsoleIntercomAdapterInit(ConsoleIntercomAdapter* pAdapter, Intercom* pIntercom, uint8_t* pBuf, uint16_t size, uint8_t source, uint8_t dest)
{
	pAdapter->pIntercom = pIntercom;
	pAdapter->pTransferBuf = pBuf;
	pAdapter->transferBufSize = size;
	pAdapter->droppedPackets = 0;
	pAdapter->intercomSource = source;
	pAdapter->intercomDestination = dest;

	IntercomClient client;
	client.pUserData = pAdapter;
	client.rxCallback = &intercomCallback;
	client.section = INTERCOM_SECTION_CONSOLE;

	IntercomAddClient(pIntercom, client);

	console.data.txCallback = &consoleTxCallback;
	console.data.pUserData = pAdapter;
}
