/*
 * critical.h
 *
 *  Created on: 19.10.2010
 *      Author: AndreR
 */

#ifndef __CRITICAL_H__
#define __CRITICAL_H__

#include "FreeRTOS.h"
#include "semphr.h"

static __inline__ void __iRestoreCritical(unsigned char* pLock)
{
	pLock = pLock;

	__asm volatile
	(
		"	mov r0, #0					\n"
		"	msr basepri, r0				\n"
		:::"r0"
	);

	__asm__ volatile("" ::: "memory");
}

static __inline__ unsigned char __iClearCritical()
{
	__asm volatile
	(
		"	mov r0, %0								\n"
		"	msr basepri, r0							\n"
		::"i"(configMAX_SYSCALL_INTERRUPT_PRIORITY):"r0"
	);

	return 1;
}

static __inline__ unsigned char __iClearCriticalTaskOnly()
{
	__asm volatile
	(
		"	mov r0, %0								\n"
		"	msr basepri, r0							\n"
		::"i"(configKERNEL_INTERRUPT_PRIORITY):"r0"
	);

	return 1;
}

#define SYSCALL_CRITICAL_SECTION() \
for(unsigned char __info __attribute((__cleanup__(__iRestoreCritical))) = __iClearCritical(); __info; __info = 0)

#define TASK_CRITICAL_SECTION() \
for(unsigned char __info __attribute((__cleanup__(__iRestoreCritical))) = __iClearCriticalTaskOnly(); __info; __info = 0)

// Atomic lock and critical section stuff
typedef struct __TaskMutexInfo
{
	uint8_t restoreState;
	uint8_t restoreState2;
	uint8_t run;
	xSemaphoreHandle handle;
	xSemaphoreHandle handle2;
} _TaskMutexInfo;

static __inline__ void __iReleaseMutex(_TaskMutexInfo* pLock)
{
	if(pLock->restoreState)
		xSemaphoreGive(pLock->handle);

	__asm__ volatile("" ::: "memory");
}

static __inline__ _TaskMutexInfo __iTakeMutex(xSemaphoreHandle handle)
{
	_TaskMutexInfo lock;
	lock.run = 1;
	lock.handle = handle;
	lock.restoreState = xSemaphoreTake(handle, portMAX_DELAY) == pdTRUE ? 1 : 0;

	return lock;
}

static __inline__ void __iReleaseMutexRecursive(_TaskMutexInfo* pLock)
{
	if(pLock->restoreState)
		xSemaphoreGiveRecursive(pLock->handle);

	__asm__ volatile("" ::: "memory");
}

static __inline__ _TaskMutexInfo __iTakeMutexRecursive(xSemaphoreHandle handle)
{
	_TaskMutexInfo lock;
	lock.run = 1;
	lock.handle = handle;
	lock.restoreState = xSemaphoreTakeRecursive(handle, portMAX_DELAY) == pdTRUE ? 1 : 0;

	return lock;
}

static __inline__ void __iReleaseMutex2(_TaskMutexInfo* pLock)
{
	if(pLock->restoreState)
		xSemaphoreGive(pLock->handle);

	if(pLock->restoreState2)
		xSemaphoreGive(pLock->handle2);

	__asm__ volatile("" ::: "memory");
}

static __inline__ _TaskMutexInfo __iTakeMutex2(xSemaphoreHandle handle, xSemaphoreHandle handle2)
{
	_TaskMutexInfo lock;
	lock.run = 1;
	lock.handle = handle;
	lock.handle2 = handle2;
	lock.restoreState = xSemaphoreTake(handle, portMAX_DELAY) == pdTRUE ? 1 : 0;
	lock.restoreState2 = xSemaphoreTake(handle2, portMAX_DELAY) == pdTRUE ? 1 : 0;

	return lock;
}

#define TASK_MUTEX(handle) \
for(_TaskMutexInfo __info __attribute((__cleanup__(__iReleaseMutex))) = __iTakeMutex(handle); __info.run; __info.run = 0)

#define TASK_MUTEX_RECURSIVE(handle) \
for(_TaskMutexInfo __info __attribute((__cleanup__(__iReleaseMutexRecursive))) = __iTakeMutexRecursive(handle); __info.run; __info.run = 0)

#define TASK_MUTEX_WITH_NAME(handle, varName) \
for(_TaskMutexInfo varName __attribute((__cleanup__(__iReleaseMutex))) = __iTakeMutex(handle); varName.run; verName.run = 0)

#define TASK_MUTEX2(handle, handle2) \
for(_TaskMutexInfo __info __attribute((__cleanup__(__iReleaseMutex2))) = __iTakeMutex2(handle, handle2); __info.run; __info.run = 0)

#endif /* CRITICAL_H_ */
