/*
 * md5.c
 *
 *  Created on: 03.06.2013
 *      Author: AndreR
 */

#include "md5.h"

#include "task.h"

#include "stm32f4xx_rcc.h"
#include "stm32f4xx_hash.h"
#include "misc.h"

#include "util/critical.h"

MD5 md5;

void HASH_RNG_IRQHandler()
{
	HASH->SR = 0;

	int32_t yield = pdFALSE;

	xSemaphoreGiveFromISR(md5.hDigestComplete, &yield);
	if(yield)
		taskYIELD();
}

void MD5Init(uint8_t priority)
{
	RCC_AHB2PeriphClockCmd(RCC_AHB2Periph_HASH, ENABLE);

	md5.hHashMutex = xSemaphoreCreateMutex();
	vSemaphoreCreateBinary(md5.hDigestComplete);

	HASH_InitTypeDef hashInit;
	hashInit.HASH_AlgoMode = HASH_AlgoMode_HASH;
	hashInit.HASH_AlgoSelection = HASH_AlgoSelection_MD5;
	hashInit.HASH_DataType = HASH_DataType_32b;

	HASH_Init(&hashInit);

	HASH_SetLastWordValidBitsNbr(0);

	NVIC_InitTypeDef nvicInit;
	nvicInit.NVIC_IRQChannel = HASH_RNG_IRQn;
	nvicInit.NVIC_IRQChannelCmd = ENABLE;
	nvicInit.NVIC_IRQChannelPreemptionPriority = priority;
	nvicInit.NVIC_IRQChannelSubPriority = 0;

	NVIC_Init(&nvicInit);

	HASH_ClearITPendingBit(HASH_IT_DCI);
	HASH_ITConfig(HASH_IT_DCI, ENABLE);
}

void MD5Hash(uint32_t* pData, uint32_t dataLength, uint32_t* pMD5)
{
	// block concurrent task access
	TASK_MUTEX(md5.hHashMutex)
	{
		HASH_Reset();

		xSemaphoreTake(md5.hDigestComplete, 0);

		for(uint32_t i = 0; i < dataLength; i++)
		{
			HASH_DataIn(*(pData+i));
		}

		HASH_StartDigest();

		xSemaphoreTake(md5.hDigestComplete, portMAX_DELAY);

		pMD5[0] = HASH->HR[0];
		pMD5[1] = HASH->HR[1];
		pMD5[2] = HASH->HR[2];
		pMD5[3] = HASH->HR[3];
	}
}
