#ifndef __PID_H__
#define __PID_H__

#include "stm32f4xx.h"

typedef int32_t(*PIDFeedbackFunc)();
typedef void(*PIDOutputFunc)(int32_t);

typedef struct _PID
{
	volatile int32_t Kp;
	volatile int32_t Ki;
	volatile int32_t Kd;

	volatile int32_t lastErrorT0;
	volatile int32_t lastErrorT1;
	volatile int32_t lastErrorT2;
	volatile int32_t lastOutput;

	volatile int32_t setpoint;

	volatile int32_t max;
	volatile int32_t slewMax;
	volatile int32_t frictionKick;

	int32_t dt;

	struct
	{
		PIDFeedbackFunc feedback;
		PIDOutputFunc output;
	} funcs;

	struct
	{
		int32_t current;
		int32_t pError;
		int32_t iError;
		int32_t dError;
		int32_t output;
		uint8_t overload;
	} log;
} PID;

void PIDUpdate(PID* pPid);
void PIDInit(PID* pPid, PIDOutputFunc outFunc, PIDFeedbackFunc feedbackFunc);
void PIDSetConfiguration(PID* pPid, int32_t kp, int32_t ki, int32_t kd, int32_t dt, int32_t max, int32_t frictionKick);
void PIDSetSetpoint(PID* pPid, int32_t sp);
void PIDSetZero(PID* pPid);

#endif
