/*
 * hal.c
 *
 *  Created on: 10.05.2011
 *      Author: AndreR
 */

#include "hal.h"
#include "spi.h"

#include <avr/interrupt.h>

/**
 * value and prescaler calculation for 8-bit timer
 *
 * @param time Time to calc  in [us]
 * @param pPrescaler prescaler output
 * @param pValue timer value output
 */
static void calcTimer8(FixedPoint142 time, uint8_t* pPrescaler, uint8_t* pValue)
{
	uint32_t time32 = time.value;

	if(time.exp == 1)
		time32 *= 10;

	if(time.exp == 2)
		time32 *= 100;

	if(time.exp == 3)
		time32 *= 1000;

	if(time32 < 21)
	{
		// 0.083us precision
		*pValue = (time32*100000)/8333;
		*pPrescaler = 1;	// 1
	}
	else if(time32 < 170)
	{
		// 0.666 us precision
		*pValue = (time32*10000)/6667;
		*pPrescaler = 2;	// 8
	}
	else if(time32 < 1360)
	{
		// 5.333us precision
		*pValue = (time32*1000)/5333;
		*pPrescaler = 3;	// 64
	}
	else if(time32 < 5440)
	{
		// 21.333us
		*pValue = (time32*100)/2133;
		*pPrescaler = 4;	// 256
	}
	else if(time32 < 21760)
	{
		// 83.333us
		*pValue = (time32*100)/8533;
		*pPrescaler = 5;	// 1024
	}
	else
	{
		*pValue = 1;
		*pPrescaler = 1;
	}
}

void EnableCharge(uint16_t onTicks, uint16_t offTicks)
{
	// init Timer 1 (CHG)
	TCCR1B = (1 << WGM13);
	TCNT1 = 0;
	OCR1A = 0;
	ICR1 = 12;	// determines period (T), in clock cycles (@12Mhz)
	TCCR1A = (1 << COM1A1); // non-inverting Phase and frequency correct PWM, TOP = ICR1
	TIMSK1 = 0;

	ICR1 = onTicks + offTicks;
	OCR1A = onTicks;

	TCCR1B |= (1 << CS10);
}

void DisableCharge()
{
	OCR1A = 0;
}

void EnableFire1(FixedPoint142 duration)
{
	uint8_t prescaler;
	uint8_t value;
	calcTimer8(duration, &prescaler, &value);

	PORTD &= ~(1 << LEDR);

	OCR0B = value;			// switch low at value
	TCNT0 = 0xFF;			// reset counter to max
	PORTD |= (1 << DIS1);
	TCCR0B = prescaler;	// and GO!
}

void EnableFire2(FixedPoint142 duration)
{
	uint8_t prescaler;
	uint8_t value;
	calcTimer8(duration, &prescaler, &value);

	PORTD &= ~(1 << LEDR);

	OCR0B = value;			// switch low at value
	TCNT0 = 0xFF;			// reset counter to max
	PORTD |= (1 << DIS2);
	TCCR0B = prescaler;	// and GO!
}

void DisableFire()
{
	PORTD &= ~((1 << DIS1) | (1 << DIS2));

	TCCR0B = 0;	// disable timer

	PORTD |= (1 << LEDR);
}

/*
static void CalcTimer(FixedPoint142 time, uint8_t* pPrescaler, uint16_t* pValue)
{
	uint32_t time32 = time.value;

	if(time.exp == 1)
		time32 *= 10;

	if(time.exp == 2)
		time32 *= 100;

	if(time.exp == 3)
		time32 *= 1000;

	if(time32 < 54610)
	{
		// 0.083us precision
		*pValue = (time32*10000)/8333;
		*pPrescaler = 1;	// 1
	}
	else if(time32 < 436906)
	{
		// 0.666 us precision
		*pValue = (time32*1000)/6667;
		*pPrescaler = 2;	// 8
	}
	else if(time32 < 3495000)
	{
		// 5.333us precision
		*pValue = (time32*100)/5333;
		*pPrescaler = 3;	// 64
	}
	else if(time32 < 13978000)
	{
		// 21.333us
		*pValue = (time32*10)/2133;
		*pPrescaler = 4;	// 256
	}
	else if(time32 < 55920000)
	{
		// 83.333us
		*pValue = (time32*10)/8533;
		*pPrescaler = 5;	// 1024
	}
	else
	{
		*pValue = 1;
		*pPrescaler = 1;
	}
}
*/
