/*
 * ctrl_fusion.h
 *
 *  Created on: 18.03.2013
 *      Author: AndreR
 */

#ifndef CTRL_FUSION_H_
#define CTRL_FUSION_H_

#include "ctrl.h"
#include "util/kf.h"
#include "util/hermite_spline.h"
#include "commands.h"

typedef struct _CtrlFusion
{
	KF kfX;
	KF kfY;
	KF kfW;

	uint32_t lastWakeTime;
	uint32_t lastReaction;

	float outMotor[4];
	arm_matrix_instance_f32 matOutMotor;

	float curMotor[4];
	arm_matrix_instance_f32 matCurMotor;

	float curXYW[3];
	arm_matrix_instance_f32 matCurXYW;

	float globalOutXYW[3];
	arm_matrix_instance_f32 matGlobalOutXYW;

	float outXYW[3];
	arm_matrix_instance_f32 matOutXYW;

	float curForce[4];
	arm_matrix_instance_f32 matCurForce;

	float curAcc[3];
	arm_matrix_instance_f32 matCurAcc;

	HermiteSplineList splineList[3];
	uint32_t trajectoryStartTimeXY; // hfTickCount
	uint32_t trajectoryStartTimeW; // hfTickCount

	uint32_t lastPositionUpdateTime; // hfTickCount
	float visionTimeout; // [s]
	uint8_t visionLost;
	uint8_t visionBack;
	uint32_t visionBackTime;

	arm_pid_instance_f32 pidPos[3];
	arm_pid_instance_f32 pidVel[3];
	arm_pid_instance_f32 pidAccMotor[4];

	uint32_t posCounter;
	uint32_t velCounter;

	float posOutput[3];
	float velOutput[3];
} CtrlFusion;

extern CtrlFusion ctrlFusion;
extern CtrlInstance ctrlFusionInstance;

void CtrlFusionSetParams(float kp, float ki, float kd);
void CtrlFusionSetStateUncertainty(float* pEx);
void CtrlFusionSetSensorUncertainty(float* pEz);
void CtrlFusionAddSplineCmdXY(CtrlSpline2D* pSpline);
void CtrlFusionAddSplineCmdW(CtrlSpline1D* pSpline);
void CtrlFusionSetFilterParams(CtrlSetFilterParams* pParams);
void CtrlFusionSetPIDParams(CtrlSetPIDParams* pParams);

#endif /* CTRL_FUSION_H_ */
