/*
 * ctrl_test_step.c
 *
 *  Created on: 16.04.2013
 *      Author: AndreR
 */

#include "ctrl_test_step.h"
#include "util/console.h"
#include "network.h"

#include <string.h>

static void init();
static void controller(CtrlInput* pInput, CtrlOutput* pOutput);

CtrlTestStep ctrlTestStep;
CtrlInstance ctrlTestStepInstance = {.pInit = &init, .pExit = 0, .pControllerFunction = &controller, .deltaT = 8}; // 250Hz

static void init()
{
	memset(&ctrlTestStep, 0, sizeof(CtrlTestStep));

	ctrlTestStep.testEndTime = xTaskGetTickCount();
}

static void controller(CtrlInput* pInput, CtrlOutput* pOutput)
{
	if(pInput->motor.updated == 0)
		return;

	pInput->motor.updated = 0;

	// calculate precise dT

	if(pInput->enc.updated)
	{
		pInput->enc.updated = 0;

		float dT = (rtosHfTickCount - ctrlTestStep.lastHfTickTime)*RTOS_HF_TICK_RATE*1e-6f;
		ctrlTestStep.lastHfTickTime = rtosHfTickCount;

		// calculate motor acceleration
		for(uint8_t i = 0; i < 4; i++)
		{
			ctrlTestStep.curAcc[i] = (pInput->enc.motorVel[i] - ctrlTestStep.lastMotor[i])/dT;
		}
	}

	// save old motor velocity
	memcpy(ctrlTestStep.lastMotor, pInput->enc.motorVel, sizeof(float)*4);

	for(uint8_t i = 0; i < 4; i++)
	{
		ctrlTestStep.estJ[i] = CTRL_MOTOR_KM*pInput->motor.current[i]/ctrlTestStep.curAcc[i];
	}

	if(ctrlTestStep.numSamples < CTRL_TEST_STEP_NUM_SAMPLES)
	{
		for(uint8_t i = 0; i < 4; i++)
		{
			ctrlTestStep.samples[ctrlTestStep.numSamples][i][0] = ctrlTestStep.curAcc[i];
			ctrlTestStep.samples[ctrlTestStep.numSamples][i][1] = pInput->motor.current[i];
		}

		ctrlTestStep.numSamples++;

		pOutput->motorPWM[0] = ctrlTestStep.stepOutput;
		pOutput->motorPWM[1] = ctrlTestStep.stepOutput;
		pOutput->motorPWM[2] = ctrlTestStep.stepOutput;
		pOutput->motorPWM[3] = ctrlTestStep.stepOutput;
	}
	else
	{
		pOutput->motorPWM[0] = 0;
		pOutput->motorPWM[1] = 0;
		pOutput->motorPWM[2] = 0;
		pOutput->motorPWM[3] = 0;
	}
}

void CtrlTestStepSet(float output)
{
	ctrlTestStep.stepOutput = output;
	ctrlTestStep.numSamples = 0;
}

void CtrlTestStepPrintData()
{
	for(uint32_t s = 0; s < CTRL_TEST_STEP_NUM_SAMPLES; s++)
	{
		NetworkPrint("I: %.3f, a: %.3f\tI: %.3f, a: %.3f\tI: %.3f, a: %.3f\tI: %.3f, a: %.3f",
				ctrlTestStep.samples[s][0][1], ctrlTestStep.samples[s][0][0],
				ctrlTestStep.samples[s][1][1], ctrlTestStep.samples[s][1][0],
				ctrlTestStep.samples[s][2][1], ctrlTestStep.samples[s][2][0],
				ctrlTestStep.samples[s][3][1], ctrlTestStep.samples[s][3][0]);

		ConsolePrint("I: %.3f, a: %.3f\tI: %.3f, a: %.3f\tI: %.3f, a: %.3f\tI: %.3f, a: %.3f\r\n",
				ctrlTestStep.samples[s][0][1], ctrlTestStep.samples[s][0][0],
				ctrlTestStep.samples[s][1][1], ctrlTestStep.samples[s][1][0],
				ctrlTestStep.samples[s][2][1], ctrlTestStep.samples[s][2][0],
				ctrlTestStep.samples[s][3][1], ctrlTestStep.samples[s][3][0]);

		vTaskDelay(20/portTICK_RATE_MS);
	}
}
