/*
 * ctrl_vel.c
 *
 *  Created on: 08.03.2013
 *      Author: AndreR
 */

#include "ctrl_vel.h"

#include <string.h>

#define REACTION_THRESHOLD 0.01f
#define STALL_TIMEOUT 5000

static void init();
static void controller(CtrlInput* pInput, CtrlOutput* pOutput);

CtrlVel ctrlVel;
CtrlInstance ctrlVelInstance = {.pInit = &init, .pExit = 0, .pControllerFunction = &controller, .deltaT = 20};

static void init()
{
	ctrlVel.lastWakeTime = xTaskGetTickCount();
	ctrlVel.lastReaction = ctrlVel.lastWakeTime;

	for(uint8_t i = 0; i < 3; i++)
	{
		ctrlVel.pid[i].Kp = 0.1f;
		ctrlVel.pid[i].Ki = 0.1f;
		ctrlVel.pid[i].Kd = 0.0f;
		arm_pid_init_f32(&ctrlVel.pid[i], 1);
	}

	memset(ctrlVel.setpoint, 0, sizeof(float)*3);

	ctrlVel.matCurMotor.pData = ctrlVel.curMotor;
	ctrlVel.matCurMotor.numCols = 1;
	ctrlVel.matCurMotor.numRows = 4;

	ctrlVel.matSetpoint.pData = ctrlVel.setpoint;
	ctrlVel.matSetpoint.numCols = 1;
	ctrlVel.matSetpoint.numRows = 3;

	ctrlVel.matCurXYW.pData = ctrlVel.curXYW;
	ctrlVel.matCurXYW.numCols = 1;
	ctrlVel.matCurXYW.numRows = 3;

	ctrlVel.matOutXYW.pData = ctrlVel.outXYW;
	ctrlVel.matOutXYW.numCols = 1;
	ctrlVel.matOutXYW.numRows = 3;

	ctrlVel.matOutMotor.pData = ctrlVel.outMotor;
	ctrlVel.matOutMotor.numCols = 1;
	ctrlVel.matOutMotor.numRows = 4;
}

static void controller(CtrlInput* pInput, CtrlOutput* pOutput)
{
	if(pInput->enc.updated == 0)
		return;

	pInput->enc.updated = 0;

	memcpy(ctrlVel.curMotor, pInput->enc.motorVel, sizeof(float)*4);

	ctrlVel.curMotor[0] *= ctrl.botParams.wheelRadius;
	ctrlVel.curMotor[1] *= ctrl.botParams.wheelRadius;
	ctrlVel.curMotor[2] *= ctrl.botParams.wheelRadius;
	ctrlVel.curMotor[3] *= ctrl.botParams.wheelRadius; // value is now speed over ground [m/s]

	arm_mat_mult_f32(&ctrl.matMotor2XYW, &ctrlVel.matCurMotor, &ctrlVel.matCurXYW);

	// matCurXYW contains X and Y in [m/s] and W in [rad/s] from encoder data

	// velocity state update
	memcpy(ctrl.state.vel, ctrlVel.curXYW, sizeof(float)*3);
	ctrl.state.updated[1] = 1;

	// do control...
	// single PID on XYW
	for(uint8_t i = 0; i < 3; i++)
	{
		float error = ctrlVel.setpoint[i] - ctrlVel.curXYW[i];

		ctrlVel.outXYW[i] = arm_pid_f32(&ctrlVel.pid[i], error);
	}

	// stall protection
	for(uint8_t i = 0; i < 3; i++)
	{
		if(ctrlVel.curXYW[i] < -REACTION_THRESHOLD || ctrlVel.curXYW[i] > REACTION_THRESHOLD)
			ctrlVel.lastReaction = ctrlVel.lastWakeTime;
	}

	if(ctrlVel.lastWakeTime - ctrlVel.lastReaction > STALL_TIMEOUT/portTICK_RATE_MS)
	{
		arm_pid_reset_f32(&ctrlVel.pid[0]);
		arm_pid_reset_f32(&ctrlVel.pid[1]);
		arm_pid_reset_f32(&ctrlVel.pid[2]);

		ctrlVel.outXYW[0] = 0.0f;
		ctrlVel.outXYW[1] = 0.0f;
		ctrlVel.outXYW[2] = 0.0f;

		ctrlVel.lastReaction = ctrlVel.lastWakeTime;
	}

	arm_mat_mult_f32(&ctrl.matXYW2Motor, &ctrlVel.matOutXYW, &ctrlVel.matOutMotor);

	// convert speed over ground to motor output
	for(uint8_t i = 0; i < 4; i++)
	{
		ctrlVel.outMotor[i] /= 5.0f;	// TODO: find correct maximum to scale output to 1.0f
	}

	memcpy(pOutput->motorPWM, ctrlVel.outMotor, sizeof(float)*4);

	ctrlVel.lastWakeTime = xTaskGetTickCount();
}

void CtrlVelSetTarget(float x, float y, float w)
{
	ctrlVel.setpoint[0] = x;
	ctrlVel.setpoint[1] = y;
	ctrlVel.setpoint[2] = w;
}
