/*
 * motor_ir.c
 *
 *  Created on: 21.12.2012
 *      Author: AndreR
 */

#include "motor_ir.h"

#include "stm32f4xx_rcc.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_tim.h"
#include "misc.h"
#include "spi1_2_3.h"
#include "atmega/motor_prog.h"
#include "util/console.h"
#include "commands.h"
#include "network.h"
#include "hal/exti.h"
#include "kicker.h"

#include "FreeRTOS.h"
#include "task.h"

#include <string.h>

#define MOVE_ATMEGA_CS (1 << 10)	// GPIOB
#define MOVE_ATMEGA_RST (1 << 11)	// GPIOB

MotorIr motorIr;

void TIM8_BRK_TIM12_IRQHandler()
{
	TIM12->SR = 0;

	int32_t yield = 0;
	xSemaphoreGiveFromISR(motorIr.hRun, &yield);
	if(yield == pdTRUE)
		taskYIELD();
}

void MotorIrInit()
{
	motorIr.cb = 0;

	GPIO_InitTypeDef gpioInit;

	gpioInit.GPIO_Mode = GPIO_Mode_OUT;
	gpioInit.GPIO_Pin = MOVE_ATMEGA_CS | MOVE_ATMEGA_RST;
	gpioInit.GPIO_Speed = GPIO_Speed_50MHz;
	gpioInit.GPIO_OType = GPIO_OType_PP;
	GPIO_Init(GPIOB, &gpioInit);

	GPIOB->BSRRL = MOVE_ATMEGA_CS | MOVE_ATMEGA_RST;

	ATMegaInit(&motorIr.atmega, &spi3, GPIOB, MOVE_ATMEGA_CS, GPIOB, MOVE_ATMEGA_RST);

	vSemaphoreCreateBinary(motorIr.hRun);

	// init precision timer for accurate controller dT
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM12, ENABLE);

	TIM_TimeBaseInitTypeDef timeBaseInit;
	timeBaseInit.TIM_ClockDivision = TIM_CKD_DIV1;
	timeBaseInit.TIM_CounterMode = TIM_CounterMode_Up;
	timeBaseInit.TIM_Period = 1;	// 1000Hz
	timeBaseInit.TIM_Prescaler = 41999;	// = 0.5ms
	timeBaseInit.TIM_RepetitionCounter = 0;
	TIM_TimeBaseInit(TIM12, &timeBaseInit);

	TIM_ARRPreloadConfig(TIM12, DISABLE);

	TIM_ITConfig(TIM12, TIM_IT_Update, ENABLE);

	NVIC_InitTypeDef nvicInit;
	nvicInit.NVIC_IRQChannel = TIM8_BRK_TIM12_IRQn;
	nvicInit.NVIC_IRQChannelCmd = ENABLE;
	nvicInit.NVIC_IRQChannelPreemptionPriority = IRQL_MOTOR_IR_TIMER;
	nvicInit.NVIC_IRQChannelSubPriority = 0;
	NVIC_Init(&nvicInit);

	TIM_Cmd(TIM12, ENABLE);
}

void MotorIrTask(void* params)
{
	params = params;

	// -- Movement
	uint8_t programUpdated  = 0;
	int16_t result = ATMegaUpdateProgram(&motorIr.atmega, motor_prog_data, MOTOR_PROG_SIZE, &programUpdated);

	ConsolePrint("--- Motor AVR ---\r\n");
	ConsolePrint("Signature: %X %X %X\r\n", motorIr.atmega.signature[0], motorIr.atmega.signature[1], motorIr.atmega.signature[2]);
	ConsolePrint("Fuses: %X %X %X\r\n", motorIr.atmega.fuses[0], motorIr.atmega.fuses[1], motorIr.atmega.fuses[2]);
	if(programUpdated)
		ConsolePrint("\e[0;36mUpdated motor program\e[0m\r\n");
	if(result)
		ConsolePrint("\e[0;31mMotor program update failed (0x%X)\e[0m\r\n", result);

	vTaskDelay(100/portTICK_RATE_MS);

	uint32_t hfTickStart = 0;

	while(1)
	{
		motorIr.runtime = (rtosHfTickCount-hfTickStart)*RTOS_HF_TICK_RATE*1e-6f;
		hfTickStart = rtosHfTickCount;

		xSemaphoreTake(motorIr.hRun, 10/portTICK_RATE_MS);

		ATMegaExchange(&motorIr.atmega, motorIr.atData.exchange.mosi, motorIr.atData.exchange.miso, MOTOR_DATA_SIZE);

		if(motorIr.cb)
			(*motorIr.cb)();

		motorIr.atData.cmd = 0;
	}
}

void MotorIrSetCallback(MotorUpdatedCallback cb)
{
	motorIr.cb = cb;
}

