/*
 * self_check.c
 *
 *  Created on: 29.12.2012
 *      Author: AndreR
 */

#include "self_check.h"

#include "FreeRTOS.h"
#include "task.h"

#include "hal/motor.h"
#include "hal/enc.h"
#include "motor_ir.h"
#include "ctrl.h"
#include "dribbler.h"

#include <string.h>

void SelfCheckMotorHall(SelfCheckMotorHallResult* pResult)
{
	if(!pResult)
		return;

	CtrlInstance* pOrigInstance = ctrl.pInstance;
	CtrlSetController(0);
	dribbler.enabled = 0;

	memset(pResult, 0, sizeof(SelfCheckMotorHallResult));

	for(uint8_t motor = 0; motor < 5; motor++)
	{
		vTaskDelay(10/portTICK_RATE_MS);

		for(uint8_t i = 1; i < 7; i++)
		{
			MotorClearDriverFault(motor);

			vTaskDelay(10/portTICK_RATE_MS);

			MotorSetPhase(motor, i, 3);
			vTaskDelay(100/portTICK_RATE_MS);

			MotorSetPhase(motor, i, 5);
			vTaskDelay(500/portTICK_RATE_MS);

			pResult->M[motor].current[i-1] = motorIr.atData.current[motor];
		}

		MotorClearDriverFault(motor);
	}

	for(uint8_t motor = 0; motor < 5; motor++)
	{
		MotorSetPower(motor, 3);

		for(uint16_t i = 0; i < 2500; i++)
		{
			uint32_t pos = MotorGetHallPosition(motor)-1;
			if(pos < 6)
				++pResult->M[motor].hallEvents[pos];

			vTaskDelay(2/portTICK_RATE_MS);
		}

		MotorSetPower(motor, 0);
	}

	CtrlSetController(pOrigInstance);
	dribbler.enabled = 1;
}

void SelfCheckGear(SelfCheckGearResult* pResult)
{
	if(!pResult)
		return;

	CtrlInstance* pOrigInstance = ctrl.pInstance;
	CtrlSetController(0);
	dribbler.enabled = 0;

	memset(pResult, 0, sizeof(SelfCheckGearResult));

	EncEnableAutoUpdate(0);

	MotorZeroAllComs();
	EncZeroAll();

	for(uint8_t motor = 0; motor < 5; motor++)
	{
		MotorClearDriverFault(motor);

		vTaskDelay(250/portTICK_RATE_MS);

		MotorSetPower(motor, 5);

		vTaskDelay(2000/portTICK_RATE_MS);

		MotorSetPower(motor, 0);

		vTaskDelay(500/portTICK_RATE_MS);

		pResult->M[motor].fwdMotorAngle = MotorGetCurrentAngle(motor);
		pResult->M[motor].fwdEncAngle = EncGetCurrentAngle(motor);
		pResult->M[motor].fwdRatio = pResult->M[motor].fwdMotorAngle/pResult->M[motor].fwdEncAngle;

		MotorZeroAllComs();
		EncZeroAll();

		MotorSetPower(motor, -5);

		vTaskDelay(2000/portTICK_RATE_MS);

		MotorSetPower(motor, 0);

		vTaskDelay(500/portTICK_RATE_MS);

		pResult->M[motor].revMotorAngle = MotorGetCurrentAngle(motor);
		pResult->M[motor].revEncAngle = EncGetCurrentAngle(motor);
		pResult->M[motor].revRatio = pResult->M[motor].revMotorAngle/pResult->M[motor].revEncAngle;
	}

	EncEnableAutoUpdate(1);

	CtrlSetController(pOrigInstance);
	dribbler.enabled = 1;
}
