/*
 * sensors.c
 *
 *  Created on: 14.03.2013
 *      Author: AndreR
 */

#include "sensors.h"

#include "stm32f4xx_rcc.h"
#include "stm32f4xx_tim.h"
#include "misc.h"

#include "spi1_2_3.h"
#include "hal/l3g.h"
#include "hal/lis3.h"
#include "constants.h"
#include "intercom1.h"
#include "intercom_constants.h"
#include "network.h"
#include "commands.h"

#include "task.h"

Sensors sensors;

void TIM1_UP_TIM10_IRQHandler()
{
	TIM10->SR = 0;

	int32_t yield = 0;
	xSemaphoreGiveFromISR(sensors.hRun, &yield);
	if(yield == pdTRUE)
		taskYIELD();
}

void SensorsInit()
{
	vSemaphoreCreateBinary(sensors.hRun);

	L3GInit(&spi2);
	LIS3Init(&spi2);

	// init precision timer for accurate controller dT
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM10, ENABLE);

	TIM_TimeBaseInitTypeDef timeBaseInit;
	timeBaseInit.TIM_ClockDivision = TIM_CKD_DIV1;
	timeBaseInit.TIM_CounterMode = TIM_CounterMode_Up;
	timeBaseInit.TIM_Period = 5;	// 400Hz
	timeBaseInit.TIM_Prescaler = 41999;	// = 0.5ms
	timeBaseInit.TIM_RepetitionCounter = 0;
	TIM_TimeBaseInit(TIM10, &timeBaseInit);

	TIM_ARRPreloadConfig(TIM10, DISABLE);

	TIM_ITConfig(TIM10, TIM_IT_Update, ENABLE);

	NVIC_InitTypeDef nvicInit;
	nvicInit.NVIC_IRQChannel = TIM1_UP_TIM10_IRQn;
	nvicInit.NVIC_IRQChannelCmd = ENABLE;
	nvicInit.NVIC_IRQChannelPreemptionPriority = IRQL_SENSORS_TIMER;
	nvicInit.NVIC_IRQChannelSubPriority = 0;
	NVIC_Init(&nvicInit);

	TIM_Cmd(TIM10, ENABLE);
}

void SensorsTask(void* params)
{
	(void) params;

	IntercomHeader iHeader;
	iHeader.address = INTERCOM_ADDRESS_MEDIA;
	iHeader.section = INTERCOM_SECTION_SENSORS;
	iHeader.command = 0;

	PacketHeader header;
	header.cmd = CMD_MOVEMENT_LIS3_LOG;
	header.len = LEN_MOVEMENT_LIS3_LOG;
	header.section = SECTION_MOVEMENT;

	MovementLis3Log log;

	uint8_t vel = 1;

	while(1)
	{
		xSemaphoreTake(sensors.hRun, portMAX_DELAY);

		L3GUpdate();
		LIS3Update();

		if(vel == 1)
		{
			log.vx = lis3.acc[0]*1e6f;
			log.vy = lis3.acc[1]*1e6f;
			log.vw = lis3.acc[2]*1e6f;
			vel = 0;
		}
		else
		{
			log.ax = lis3.acc[0]*1e6f;
			log.ay = lis3.acc[1]*1e6f;
			log.aw = lis3.acc[2]*1e6f;

			vel = 1;
		}

//		if(vel == 1)
//			NetworkSendPacket(&header, (uint8_t*)&log);

//		IntercomSend(&intercom1, &iHeader, sizeof(float)*3, (uint8_t*)lis3.acc);
	}
}
