/*
 * gui.h
 *
 *  Created on: 06.01.2013
 *      Author: AndreR
 */

#ifndef GUI_H_
#define GUI_H_

#include "stm32f4xx.h"

#include "primitives.h"

#define NUM_PAD_INPUT_X_POS 250
#define MAX_NUM_BTNS 16
#define MAX_BTNS 10

#define ELEMENT_TYPE_TEXT				0
#define ELEMENT_TYPE_INPUT_FLOAT		1
#define ELEMENT_TYPE_INPUT_POS_FLOAT	2
#define ELEMENT_TYPE_INPUT_INT			3
#define ELEMENT_TYPE_INPUT_POS_INT		4
#define ELEMENT_TYPE_BUTTON				5
#define ELEMENT_TYPE_BUTTON_ICON		6

#define MAX_TAB_NUM 10
#define MAX_TAB_CONTENT 15
#define POSITIVE_INTEGER 4
#define INTEGER 3
#define FLOAT 1
#define POSITIVE_FLOAT 2
#define TAB_BAR_WIDTH 32
#define CURR_TAB (*(gui.pTabs[gui.activeTab]))
#define CURR_ELEMENT (   (*(gui.pTabs[gui.activeTab])).pElements[i]   )  //CURR_TAB.elements[i]
#define IS_IN_AREA(x, y, x1, y1, x2, y2) (x >= x1 && x <= x2 && y >= y1 && y<= y2)
#define NUM_PAD_INPUT_X_POS 250

typedef void (* buttonCB)(uint8_t event);

typedef struct _GUIElement
{
	uint8_t changed;
	PointU16 pos;
	uint32_t bGColor;
	uint32_t fGColor;
	uint8_t type;

	union
	{
		struct _text
		{
			const char* pText;
		} text;

		struct _inputFieldFloat
		{
			uint16_t width;
			float* pValue;
			uint8_t placesBeforeDecPt;
			uint8_t placesAfterDecPt;   // 4 Bits each would do too, but button takes 96 Bits anyway
		} inputFieldFloat;

		struct _inputFieldInt
		{
			uint16_t width;
			int32_t* pValue;
		} inputFieldInt;

		struct _inputFieldUInt
		{
			uint16_t width;
			uint32_t* pValue;
		} inputFieldUInt;

		struct _button
		{
			SizeU16 size;
			const char* pText;
			buttonCB cb;
		} button;

		struct _buttonIcon
		{
			SizeU16 size;
			// TODO: Icon
		} buttonIcon;
	};
} GUIElement;

typedef struct _GUITab
{
	GUIElement* pElements;
	const char* pTitle;
	uint8_t numElements;
} GUITab;

typedef struct _GUI
{
	GUITab* pTabs[MAX_TAB_NUM];
	uint8_t activeTab;
	uint8_t numberOfTabs;
} GUI;

uint8_t GUIAddTab(GUITab* pTab);
void GUIInit(void);
void GUIHandler(void);
void GUIDrawTabContent(void);

#endif /* GUI_H_ */
