/*
 * fsmc.c
 *
 *  Created on: 16.01.2013
 *      Author: AndreR
 */

#include "stm32f4xx.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_fsmc.h"
#include "stm32f4xx_gpio.h"

#include "FreeRTOS.h"
#include "task.h"

void FSMCInit()
{
	GPIO_InitTypeDef gpioInit;

	/* Enable FSMC clock */
	RCC_AHB3PeriphClockCmd(RCC_AHB3Periph_FSMC, ENABLE);

	/*-- GPIOs Configuration -----------------------------------------------------*/
	/*
	 +-------------------+--------------------+------------------+------------------+
	 | PD0  <-> FSMC_D2  | PE0  <-> FSMC_NBL0 | PF0 <-> FSMC_A0  | PG0 <-> FSMC_A10 |
	 | PD1  <-> FSMC_D3  | PE1  <-> FSMC_NBL1 | PF1 <-> FSMC_A1  | PG1 <-> FSMC_A11 |
	 | PD4  <-> FSMC_NOE | PE3  <-> FSMC_A19  | PF2 <-> FSMC_A2  | PG2 <-> FSMC_A12 |
	 | PD5  <-> FSMC_NWE | PE7  <-> FSMC_D4   | PF3 <-> FSMC_A3  | PG3 <-> FSMC_A13 |
	 | PD7  <-> FSMC_NE1 | PE8  <-> FSMC_D5   | PF3 <-> FSMC_A3  | PG3 <-> FSMC_A13 |
	 | PD8  <-> FSMC_D13 | PE9  <-> FSMC_D6   | PF4 <-> FSMC_A4  | PG4 <-> FSMC_A14 |
	 | PD9  <-> FSMC_D14 | PE10 <-> FSMC_D7   | PF5 <-> FSMC_A5  | PG5 <-> FSMC_A15 |
	 | PD10 <-> FSMC_D15 | PE11 <-> FSMC_D8   | PF12 <-> FSMC_A6 | PG9 <-> FSMC_NE2 |
	 | PD11 <-> FSMC_A16 | PE12 <-> FSMC_D9   | PF13 <-> FSMC_A7 |------------------+
	 | PD12 <-> FSMC_A17 | PE13 <-> FSMC_D10  | PF14 <-> FSMC_A8 |
	 | PD13 <-> FSMC_A18 | PE14 <-> FSMC_D11  | PF15 <-> FSMC_A9 |
	 | PD14 <-> FSMC_D0  | PE15 <-> FSMC_D12  |------------------+
	 | PD15 <-> FSMC_D1  +--------------------+
	 +-------------------+
	 */

	/* GPIOD configuration */
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource0, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource1, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource4, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource5, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource7, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource8, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource9, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource10, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource11, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource12, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource13, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource14, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOD, GPIO_PinSource15, GPIO_AF_FSMC);

	gpioInit.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_4
			| GPIO_Pin_5 | GPIO_Pin_7 | GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_11
			| GPIO_Pin_12 | GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;
	gpioInit.GPIO_Mode = GPIO_Mode_AF;
	gpioInit.GPIO_Speed = GPIO_Speed_100MHz;
	gpioInit.GPIO_OType = GPIO_OType_PP;
	gpioInit.GPIO_PuPd = GPIO_PuPd_NOPULL;

	GPIO_Init(GPIOD, &gpioInit);

	/* GPIOE configuration */
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource0, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource1, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource3, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource7, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource8, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource9, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource10, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource11, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource12, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource13, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource14, GPIO_AF_FSMC);
	GPIO_PinAFConfig(GPIOE, GPIO_PinSource15, GPIO_AF_FSMC);

	gpioInit.GPIO_Pin =
			GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_3 | GPIO_Pin_7
			| GPIO_Pin_8 | GPIO_Pin_9 | GPIO_Pin_10 | GPIO_Pin_11 | GPIO_Pin_12
			| GPIO_Pin_13 | GPIO_Pin_14 | GPIO_Pin_15;

	GPIO_Init(GPIOE, &gpioInit);

	/* GPIOF configuration */
	GPIO_PinAFConfig(GPIOF, GPIO_PinSource0, GPIO_AF_FSMC );
	GPIO_PinAFConfig(GPIOF, GPIO_PinSource1, GPIO_AF_FSMC );
	GPIO_PinAFConfig(GPIOF, GPIO_PinSource2, GPIO_AF_FSMC );
	GPIO_PinAFConfig(GPIOF, GPIO_PinSource3, GPIO_AF_FSMC );
	GPIO_PinAFConfig(GPIOF, GPIO_PinSource4, GPIO_AF_FSMC );
	GPIO_PinAFConfig(GPIOF, GPIO_PinSource5, GPIO_AF_FSMC );
	GPIO_PinAFConfig(GPIOF, GPIO_PinSource12, GPIO_AF_FSMC );
	GPIO_PinAFConfig(GPIOF, GPIO_PinSource13, GPIO_AF_FSMC );
	GPIO_PinAFConfig(GPIOF, GPIO_PinSource14, GPIO_AF_FSMC );
	GPIO_PinAFConfig(GPIOF, GPIO_PinSource15, GPIO_AF_FSMC );

	gpioInit.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2
			| GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_12 | GPIO_Pin_13
			| GPIO_Pin_14 | GPIO_Pin_15;

	GPIO_Init(GPIOF, &gpioInit);

	/* GPIOG configuration */
	GPIO_PinAFConfig(GPIOG, GPIO_PinSource0, GPIO_AF_FSMC );
	GPIO_PinAFConfig(GPIOG, GPIO_PinSource1, GPIO_AF_FSMC );
	GPIO_PinAFConfig(GPIOG, GPIO_PinSource2, GPIO_AF_FSMC );
	GPIO_PinAFConfig(GPIOG, GPIO_PinSource3, GPIO_AF_FSMC );
	GPIO_PinAFConfig(GPIOG, GPIO_PinSource4, GPIO_AF_FSMC );
	GPIO_PinAFConfig(GPIOG, GPIO_PinSource5, GPIO_AF_FSMC );
	GPIO_PinAFConfig(GPIOG, GPIO_PinSource9, GPIO_AF_FSMC );

	gpioInit.GPIO_Pin = GPIO_Pin_0 | GPIO_Pin_1 | GPIO_Pin_2
			| GPIO_Pin_3 | GPIO_Pin_4 | GPIO_Pin_5 | GPIO_Pin_9;

	GPIO_Init(GPIOG, &gpioInit);

/*	gpioInit.GPIO_Mode = GPIO_Mode_OUT;
	gpioInit.GPIO_OType = GPIO_OType_PP;
	gpioInit.GPIO_Pin = GPIO_Pin_9;	// FSMC_NE2
	gpioInit.GPIO_PuPd = GPIO_PuPd_UP;
	gpioInit.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOG, &gpioInit);

	GPIOG->BSRRL = GPIO_Pin_9;

	while(1)
	{
		vTaskDelay(2/portTICK_RATE_MS);

		GPIOG->ODR ^= GPIO_Pin_9;
	}*/
}
