/*
 * sram.c
 *
 *  Created on: 01.12.2012
 *      Author: AndreR
 */

#include "sram.h"

#include "stm32f4xx.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_fsmc.h"
#include "stm32f4xx_gpio.h"

void SRAMInit()
{
	FSMC_NORSRAMInitTypeDef fsmcInit;
	FSMC_NORSRAMTimingInitTypeDef fsmcTiming;

	/*-- FSMC Configuration ------------------------------------------------------*/
	fsmcTiming.FSMC_AddressSetupTime = 2;	// 2
	fsmcTiming.FSMC_AddressHoldTime = 0;
	fsmcTiming.FSMC_DataSetupTime = 1;		// 1
	fsmcTiming.FSMC_BusTurnAroundDuration = 1;	// 1
	fsmcTiming.FSMC_CLKDivision = 0;
	fsmcTiming.FSMC_DataLatency = 0;
	fsmcTiming.FSMC_AccessMode = FSMC_AccessMode_A;

	fsmcInit.FSMC_Bank = FSMC_Bank1_NORSRAM2;
	fsmcInit.FSMC_DataAddressMux = FSMC_DataAddressMux_Disable;
	fsmcInit.FSMC_MemoryType = FSMC_MemoryType_SRAM;
	fsmcInit.FSMC_MemoryDataWidth = FSMC_MemoryDataWidth_16b;
	fsmcInit.FSMC_BurstAccessMode = FSMC_BurstAccessMode_Disable;
	fsmcInit.FSMC_AsynchronousWait = FSMC_AsynchronousWait_Disable;
	fsmcInit.FSMC_WaitSignalPolarity =	FSMC_WaitSignalPolarity_Low;
	fsmcInit.FSMC_WrapMode = FSMC_WrapMode_Disable;
	fsmcInit.FSMC_WaitSignalActive = FSMC_WaitSignalActive_BeforeWaitState;
	fsmcInit.FSMC_WriteOperation = FSMC_WriteOperation_Enable;
	fsmcInit.FSMC_WaitSignal = FSMC_WaitSignal_Disable;
	fsmcInit.FSMC_ExtendedMode = FSMC_ExtendedMode_Disable;
	fsmcInit.FSMC_WriteBurst = FSMC_WriteBurst_Disable;
	fsmcInit.FSMC_ReadWriteTimingStruct = &fsmcTiming;
	fsmcInit.FSMC_WriteTimingStruct = &fsmcTiming;

	FSMC_NORSRAMInit(&fsmcInit);

	/*!< Enable FSMC Bank1_SRAM2 Bank */
	FSMC_NORSRAMCmd(FSMC_Bank1_NORSRAM2, ENABLE);
}

void SRAMWriteBuffer(uint16_t* pBuffer, uint32_t WriteAddr, uint32_t NumHalfwordToWrite)
{
	for (; NumHalfwordToWrite != 0; NumHalfwordToWrite--) /* while there is data to write */
	{
		/* Transfer data to the memory */
		*(uint16_t *) (EXT_SRAM_ADDR + WriteAddr) = *pBuffer++;

		/* Increment the address*/
		WriteAddr += 2;
	}
}

void SRAMReadBuffer(uint16_t* pBuffer, uint32_t ReadAddr, uint32_t NumHalfwordToRead)
{
	for (; NumHalfwordToRead != 0; NumHalfwordToRead--) /* while there is data to read */
	{
		/* Read a half-word from the memory */
		*pBuffer++ = *(__IO uint16_t*) (EXT_SRAM_ADDR + ReadAddr);

		/* Increment the address*/
		ReadAddr += 2;
	}
}
