/*
 * usbh_msc.h
 *
 *  Created on: 25.01.2013
 *      Author: rYan
 */

#ifndef USBH_MSC_H_
#define USBH_MSC_H_

#include "stm32f4xx.h"

#include "usbh_core.h"
#include "usbh_stdreq.h"
#include "usbh_ioreq.h"
#include "usbh_hcs.h"

#ifdef USE_USB_OTG_FS
#define USBH_MSC_MPS_SIZE                 0x40
#else
#define USBH_MSC_MPS_SIZE                 0x200
#endif

#define USBH_MSC_BOT_CBW_SIGNATURE        0x43425355
#define USBH_MSC_BOT_CBW_TAG              0x20304050
#define USBH_MSC_BOT_CSW_SIGNATURE        0x53425355

#define USBH_MSC_ERROR_RETRY_LIMIT			10

typedef struct _MSC_Setup
{
	uint8_t hc_num_in;
	uint8_t hc_num_out;
	uint8_t bulkOutEp;
	uint8_t bulkInEp;
	uint16_t bulkInEpSize;
	uint16_t bulkOutEpSize;
	uint8_t buff[USBH_MSC_MPS_SIZE];
	uint8_t maxLun;
} MSC_Setup;

typedef enum
{
	USBH_MSC_OK = 0,
	USBH_MSC_FAIL = 1,
	USBH_MSC_PHASE_ERROR = 2,
	USBH_MSC_BUSY = 3
} USBH_MSC_Status_TypeDef;

typedef struct __MassStorageParameter
{
	uint32_t capacity;
	uint32_t senseKey;
	uint16_t pageLength;
	uint8_t writeProtect;
} MassStorageParameter_TypeDef;

typedef union _USBH_CBW_Block
{
	struct __CBW
	{
		uint32_t CBWSignature;
		uint32_t CBWTag;
		uint32_t CBWTransferLength;
		uint8_t CBWFlags;
		uint8_t CBWLUN;
		uint8_t CBWLength;
		uint8_t CBWCB[16];
	} field;

	uint8_t CBWArray[31];
} USBH_CBW_Block;

typedef union _USBH_CSW_Block
{
	struct __CSW
	{
		uint32_t CSWSignature;
		uint32_t CSWTag;
		uint32_t CSWDataResidue;
		uint8_t CSWStatus;
	} field;

	uint8_t CSWArray[13];
} USBH_CSW_Block;

typedef enum
{
	USBH_MSC_BOT_INIT_STATE = 0,
	USBH_MSC_BOT_RESET,
	USBH_MSC_GET_MAX_LUN,
	USBH_MSC_TEST_UNIT_READY,
	USBH_MSC_READ_CAPACITY10,
	USBH_MSC_MODE_SENSE6,
	USBH_MSC_REQUEST_SENSE,
	USBH_MSC_BOT_USB_TRANSFERS,
	USBH_MSC_INIT_FILESYSTEM,
	USBH_MSC_IDLE,
	USBH_MSC_CTRL_ERROR_STATE,
	USBH_MSC_UNRECOVERED_STATE
} MSCState;

typedef enum _USBH_MSC_BOT_State
{
	BOT_STATE_SEND_CBW = 0,
	BOT_STATE_DATA_IN,
	BOT_STATE_DATA_OUT,
	BOT_STATE_SEND_CSW,
	BOT_STATE_COMPLETE,
} USBH_MSC_BOT_State;

typedef struct _USBH_MSC
{
	__attribute__((aligned (4))) MSC_Setup setup;

	uint8_t errorCount;
	MassStorageParameter_TypeDef storageParams;
	USBH_Class_cb_TypeDef callbacks;

	MSCState state;
	MSCState stateBkp;

	struct
	{
		__attribute__((aligned (4))) USBH_CBW_Block CBWData;
		__attribute__((aligned (4))) USBH_CSW_Block CSWData;
		USBH_MSC_BOT_State transferState;
		URB urb;
	} bot;

	struct
	{
		__attribute__((aligned (4))) uint8_t dataInBuffer[512];
	} scsi;
} USBH_MSC;

extern USBH_MSC usbhMsc;

void USBH_MSC_Init(USB_OTG_CORE_HANDLE *pdev);

// BOT
int16_t USBH_MSC_BOTReset(USB_OTG_CORE_HANDLE *pdev, USBH_HOST *phost);
int16_t USBH_MSC_BOTTransfer(USB_OTG_CORE_HANDLE *pdev, USBH_HOST *phost, USBH_CBW_Block* pCBW, uint8_t* pData, USBH_CSW_Block* pCSW);

// SCSI
uint8_t USBH_MSC_TestUnitReady(USB_OTG_CORE_HANDLE *pdev, USBH_HOST *phost);
uint8_t USBH_MSC_ReadCapacity10(USB_OTG_CORE_HANDLE *pdev, USBH_HOST *phost);
uint8_t USBH_MSC_ModeSense6(USB_OTG_CORE_HANDLE *pdev, USBH_HOST *phost);
uint8_t USBH_MSC_RequestSense(USB_OTG_CORE_HANDLE *pdev, USBH_HOST *phost);
uint8_t USBH_MSC_Write10(USB_OTG_CORE_HANDLE *pdev, USBH_HOST *phost, uint8_t *, uint32_t, uint32_t);
uint8_t USBH_MSC_Read10(USB_OTG_CORE_HANDLE *pdev, USBH_HOST *phost, uint8_t *, uint32_t, uint32_t);
void USBH_MSC_StateMachine(USB_OTG_CORE_HANDLE *pdev, USBH_HOST *phost);

#endif /* USBH_MSC_H_ */
