/** 
 ******************************************************************************
 * @file    usbh_ioreq.c
 * @author  MCD Application Team
 * @version V2.1.0
 * @date    19-March-2012
 * @brief   This file handles the issuing of the USB transactions
 ******************************************************************************
 * @attention
 *
 * <h2><center>&copy; COPYRIGHT 2012 STMicroelectronics</center></h2>
 *
 * Licensed under MCD-ST Liberty SW License Agreement V2, (the "License");
 * You may not use this file except in compliance with the License.
 * You may obtain a copy of the License at:
 *
 *        http://www.st.com/software_license_agreement_liberty_v2
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ******************************************************************************
 */
/* Includes ------------------------------------------------------------------*/

#include "usbh_ioreq.h"

#include "usb_hcd.h"

#include "errors.h"

#define URB_TIMEOUT_MS (500/portTICK_RATE_MS)

int16_t USBHCtrlReq(USB_OTG_CORE_HANDLE* pDev, USBH_HOST* pHost, USB_Setup_TypeDef* pSetup, uint8_t* pData)
{
	if(HCDIsPipeActive(pDev, USBH_EP0_EP_NUM_OUT) || HCDIsPipeActive(pDev, USBH_EP0_EP_NUM_IN))
		return ERROR_USB_PIPE_ACTIVE;

	for(pHost->Control.errorcount = 0; pHost->Control.errorcount < 3; pHost->Control.errorcount++)
	{
		URBInit(&pHost->Control.urb, USBH_EP0_EP_NUM_OUT, pSetup->d8, USBH_SETUP_PKT_SIZE, URB_FLAG_SETUP | URB_FLAG_SYNC);
		pHost->Control.urb.syncTimeout = URB_TIMEOUT_MS;
		HCDSubmitURB(pDev, &pHost->Control.urb);

		if(pHost->Control.urb.state != URB_DONE)
			continue;

		uint8_t direction = (pSetup->b.bmRequestType & USB_REQ_DIR_MASK);

		// check if there is a data stage
		if(pSetup->b.wLength.w != 0)
		{
			if(direction == USB_D2H)
			{
				// Data Direction is IN
				URBInit(&pHost->Control.urb, USBH_EP0_EP_NUM_IN, pData, pSetup->b.wLength.w, URB_FLAG_DATA1 | URB_FLAG_SYNC);
				pHost->Control.urb.syncTimeout = URB_TIMEOUT_MS;
				HCDSubmitURB(pDev, &pHost->Control.urb);
				if(pHost->Control.urb.state != URB_DONE)
					continue;

				for(uint32_t statRetries = 0; statRetries < 500; statRetries++)
				{
					URBInit(&pHost->Control.urb, USBH_EP0_EP_NUM_OUT, 0, 0, URB_FLAG_DATA1 | URB_FLAG_SYNC);
					pHost->Control.urb.syncTimeout = URB_TIMEOUT_MS;
					HCDSubmitURB(pDev, &pHost->Control.urb);
					if(pHost->Control.urb.state != URB_NOTREADY)
						break;
				}

				if(pHost->Control.urb.state != URB_DONE)
					continue;
			}
			else
			{
				// Data Direction is OUT
				URBInit(&pHost->Control.urb, USBH_EP0_EP_NUM_OUT, pData, pSetup->b.wLength.w, URB_FLAG_DATA1 | URB_FLAG_SYNC);
				pHost->Control.urb.syncTimeout = URB_TIMEOUT_MS;
				HCDSubmitURB(pDev, &pHost->Control.urb);
				if(pHost->Control.urb.state != URB_DONE)
					continue;

				for(uint32_t statRetries = 0; statRetries < 500; statRetries++)
				{
					URBInit(&pHost->Control.urb, USBH_EP0_EP_NUM_IN, 0, 0, URB_FLAG_DATA1 | URB_FLAG_SYNC);
					pHost->Control.urb.syncTimeout = URB_TIMEOUT_MS;
					HCDSubmitURB(pDev, &pHost->Control.urb);
					if(pHost->Control.urb.state != URB_NOTREADY)
						break;
				}

				if(pHost->Control.urb.state != URB_DONE)
					continue;
			}
		}
		else	// No DATA stage
		{
			// single IN transaction
			for(uint32_t statRetries = 0; statRetries < 500; statRetries++)
			{
				URBInit(&pHost->Control.urb, USBH_EP0_EP_NUM_IN, 0, 0, URB_FLAG_DATA1 | URB_FLAG_SYNC);
				pHost->Control.urb.syncTimeout = URB_TIMEOUT_MS;
				HCDSubmitURB(pDev, &pHost->Control.urb);
				if(pHost->Control.urb.state != URB_NOTREADY)
					break;
			}

			if(pHost->Control.urb.state != URB_DONE)
				continue;
		}

		break;
	}

	if(pHost->Control.errorcount == 3)
		return ERROR_USB_CTRL_FAILED;

	return 0;
}
